<?php

/* Start the engine */
include_once( get_template_directory() . '/lib/init.php' );

/* Include code to add post templates */
include_once( get_stylesheet_directory() . '/lib/customize.php' );

/* Include code to add post templates */
include_once( get_stylesheet_directory() . '/lib/post_templates.php' );

/* Set Localization */
load_child_theme_textdomain( 'imagely-ansel', apply_filters( 'child_theme_textdomain', get_stylesheet_directory() . '/languages', 'imagely-ansel' ) );

/* Child theme */
define( 'CHILD_THEME_NAME', 'Imagely Ansel' );
define( 'CHILD_THEME_URL', 'www.imagely.com/wordpress-photography-themes/ansel' );
define( 'CHILD_THEME_VERSION', '1.0.0' );

/* Add HTML5 markup structure */
add_theme_support( 'html5', array( 'search-form', 'comment-form', 'comment-list', 'gallery', 'caption' ) );

/* Add viewport meta tag for mobile browsers */
add_theme_support( 'genesis-responsive-viewport' );

/* Remove site layouts */
/* @WP: Remove content-sidebar & full-width-content layouts to re-register with new icon */
genesis_unregister_layout( 'sidebar-content' );
genesis_unregister_layout( 'content-sidebar-sidebar' );
genesis_unregister_layout( 'sidebar-content-sidebar' );
genesis_unregister_layout( 'sidebar-sidebar-content' );
genesis_unregister_layout( 'content-sidebar' );
genesis_unregister_layout( 'full-width-content' );

/* WP: Re-register content-sidebar layout */

genesis_register_layout( 'content-sidebar', array(
    'label' => 'Right Sidebar',
    'img' => CHILD_URL . '/images/layouts/right-sidebar.png',
) );

/* WP: Register no-sidebar layout */

genesis_register_layout( 'no-sidebar', array(
    'label' => 'No Sidebar',
    'img' => CHILD_URL . '/images/layouts/no-sidebar.png',
) );

add_action('genesis_before', 'gt_new_custom_layout_logic');
function gt_new_custom_layout_logic() {
 
    $site_layout = genesis_site_layout();
     
    if ( $site_layout == 'no-sidebar' ) {
                // Remove default genesis sidebars
        remove_action( 'genesis_after_content', 'genesis_get_sidebar' );
        remove_action( 'genesis_after_content_sidebar_wrap', 'genesis_get_sidebar_alt');    
    }
}

/* WP: Re-register full-width-content layout */

genesis_register_layout( 'full-width-content', array(
    'label' => 'Full Width',
    'img' => CHILD_URL . '/images/layouts/full-width.png',
) );

/* Accessbility (not for menus because superfish interferes with responsive menu) */
add_theme_support( 'genesis-accessibility', array( '404-page', 'headings', 'search-form', 'skip-links' ) );
add_filter( 'genesis_attr_author-archive-description', 'genesis_attributes_screen_reader_class' );

/* Scripts and styles */
add_action( 'wp_enqueue_scripts', 'imagely_scripts_styles' );
function imagely_scripts_styles() {
	
	wp_enqueue_script( 'imagely-responsive-menu', get_stylesheet_directory_uri() . '/js/imagely-responsive-menu.js', array( 'jquery' ), '1.0.0', true );
	$output = array(
		'mainMenu' => __( 'Menu', 'imagely-ansel' ),
		'subMenu'  => __( 'Menu', 'imagely-ansel' ),
	);
	wp_localize_script( 'imagely-responsive-menu', 'ImagelyL10n', $output );

}

add_action( 'wp_enqueue_scripts', 'imagely_fonts' );
function imagely_fonts() {

	wp_enqueue_style( 'google-fonts', '//fonts.googleapis.com/css?family=Lato:300,300italic,400,400italic,700,900', array(), CHILD_THEME_VERSION );
	wp_enqueue_style( 'font-awesome', '//maxcdn.bootstrapcdn.com/font-awesome/latest/css/font-awesome.min.css' );
	
}

/* Add support for custom header */
/* @WP: Changed width from 800; changed height from 100 */
add_theme_support( 'custom-header', array(
	'width'           => 360,
	'height'          => 76,
	'header-selector' => '.site-title a',
	'header-text'     => false,
	'flex-height'     => true,
) );

/* Rename primary and secondary navigation menus */
add_theme_support ( 'genesis-menus' , array ( 'primary' => __( 'Header Menu', 'imagely-ansel' ), 'secondary' => __( 'Footer Menu', 'imagely-ansel' ) ) );

/* Reposition primary navigation menu */
remove_action( 'genesis_after_header', 'genesis_do_nav' );
add_action( 'genesis_header', 'genesis_do_nav', 12 );

/* Remove output of primary navigation right extras */
remove_filter( 'genesis_nav_items', 'genesis_nav_right', 10, 2 );
remove_filter( 'wp_nav_menu_items', 'genesis_nav_right', 10, 2 );

/* Reposition the secondary navigation menu */
remove_action( 'genesis_after_header', 'genesis_do_subnav' );
add_action( 'genesis_footer', 'genesis_do_subnav', 12 );

/* Reduce the secondary navigation menu to one level depth */
add_filter( 'wp_nav_menu_args', 'imagely_secondary_menu_args' );
function imagely_secondary_menu_args( $args ){

	if( 'secondary' != $args['theme_location'] )
	return $args;

	$args['depth'] = 1;
	return $args;

}

/* Remove navigation meta box */
add_action( 'genesis_theme_settings_metaboxes', 'imagely_remove_genesis_metaboxes' );
function imagely_remove_genesis_metaboxes( $_genesis_theme_settings_pagehook ) {

    remove_meta_box( 'genesis-theme-settings-nav', $_genesis_theme_settings_pagehook, 'main' );

}

/* Remove header right widget area */
unregister_sidebar( 'header-right' );

//* @WP: Unregister secondary sidebar
unregister_sidebar( 'sidebar-alt' );

/* Remove titles from blog, category, and other archive pages */
remove_action( 'genesis_before_loop', 'genesis_do_taxonomy_title_description', 15 );
remove_action( 'genesis_before_loop', 'genesis_do_taxonomy_title_description', 15 );
remove_action( 'genesis_before_loop', 'genesis_do_author_title_description', 15 );
remove_action( 'genesis_before_loop', 'genesis_do_cpt_archive_title_description' );
remove_action( 'genesis_before_loop', 'genesis_do_date_archive_title' );
remove_action( 'genesis_before_loop', 'genesis_do_blog_template_heading' );
remove_action( 'genesis_before_loop', 'genesis_do_posts_page_heading' );

/* Modify the entry title text */
function imagely_title( $title ) {
	if( genesis_get_custom_field( 'large_title' ) ) {
		$title = '<span class="imagely-large-text">' . genesis_get_custom_field( 'large_title' ) . '</span><span class="intro">' . $title . '</span>';
	}
	return $title;
}

/* Add entry title filter to posts and pages */
add_action( 'genesis_entry_header', 'imagely_add_title_filter', 1 );
function imagely_add_title_filter() {
	if( is_singular() ) {
		add_filter( 'the_title', 'imagely_title' );
	}
}

/* Remove post and page title filter after entry header */
add_action( 'genesis_entry_header', 'imagely_remove_title_filter', 15 );
function imagely_remove_title_filter() {
	remove_filter( 'the_title', 'imagely_title' );
}

// Edit the read more link text */
add_filter( 'excerpt_more' , 'imagely_read_more_link' );
add_filter( 'get_the_content_more_link' , 'imagely_read_more_link' );
add_filter( 'the_content_more_link' , 'imagely_read_more_link' );
function imagely_read_more_link() {
	return '...<p class="more-link-wrap"><a class="more-link" href="' . get_permalink() . '">' . __( 'Read More' , 'text-domain' ) .'</a></p>';
}

/* Add Imagely featured image sizes */
add_image_size( 'imagely-featured', 1280, 640, TRUE );
add_image_size( 'imagely-square', 640, 640, TRUE );

/* Replace .alignnone with .aligncenter when Align None is selected for featured images */
add_filter( 'genesis_attr_entry-image', 'imagely_replace_image_alignment' );
function imagely_replace_image_alignment( $attributes ) {
	if ( strpos($attributes['class'],'alignleft') === false && strpos($attributes['class'],'alignleft') === false ) {
		$attributes['class'] .= ' aligncenter';
	}
	return $attributes;
}

/* Modify size of the Gravatar in the author box */
add_filter( 'genesis_author_box_gravatar_size', 'imagely_author_box_gravatar' );
function imagely_author_box_gravatar( $size ) {

	return 160;

}

/* Modify size of the Gravatar in the entry comments */
add_filter( 'genesis_comment_list_args', 'imagely_comments_gravatar' );
function imagely_comments_gravatar( $args ) {

	$args['avatar_size'] = 120;
	return $args;

}

/* Remove the entry meta in the entry footer on category pages */
add_action( 'genesis_before_entry', 'imagely_remove_entry_footer' );
function imagely_remove_entry_footer() {

	if ( is_front_page() || is_archive() || is_page_template( 'page_blog.php' ) ) {

		remove_action( 'genesis_entry_footer', 'genesis_entry_footer_markup_open', 5 );
		remove_action( 'genesis_entry_footer', 'genesis_post_meta' );
		remove_action( 'genesis_entry_footer', 'genesis_entry_footer_markup_close', 15 );

	}

}

/* Setup widget counts */
function imagely_count_widgets( $id ) {

	global $sidebars_widgets;

	if ( isset( $sidebars_widgets[ $id ] ) ) {
		return count( $sidebars_widgets[ $id ] );
	}

}

/* Flexible widget classes */
function imagely_widget_area_class( $id ) {

	$count = imagely_count_widgets( $id );

	$class = '';
	
	if( $count == 1 ) {
		$class .= ' widget-full';
	} elseif( $count % 3 == 1 ) {
		$class .= ' widget-thirds';
	} elseif( $count % 4 == 1 ) {
		$class .= ' widget-fourths';
	} elseif( $count % 2 == 0 ) {
		$class .= ' widget-halves uneven';
	} else {	
		$class .= ' widget-halves';
	}
	return $class;
	
}

/* Add support for 3-column footer widget */
add_theme_support( 'genesis-footer-widgets', 3 );

/* Register widget areas */
/* @WP: Removed code to register custom home page widgets */

/* Theme Setting Defaults */
/* @WP: Changed default site layout from full-width-content in both sections below */
add_filter( 'genesis_theme_settings_defaults', 'imagely_theme_defaults' );
function imagely_theme_defaults( $defaults ) {
	$defaults['blog_cat_num']              = 10;
	$defaults['content_archive']           = 'full';
	$defaults['content_archive_limit']     = 0;
	$defaults['content_archive_thumbnail'] = 0;
	$defaults['posts_nav']                 = 'numeric';
	$defaults['site_layout']               = 'content-sidebar';
	return $defaults;
}
add_action( 'after_switch_theme', 'imagely_theme_setting_defaults' );
function imagely_theme_setting_defaults() {
	if( function_exists( 'genesis_update_settings' ) ) {
		genesis_update_settings( array(
			'blog_cat_num'              => 10,	
			'content_archive'           => 'full',
			'content_archive_limit'     => 0,
			'content_archive_thumbnail' => 0,
			'posts_nav'                 => 'numeric',
			'site_layout'               => 'content-sidebar',
		) );
	} 
	update_option( 'posts_per_page', 10 );
}

/* Simple Social Icon Defaults */
add_filter( 'simple_social_default_styles', 'imagely_social_default_styles' );
function imagely_social_default_styles( $defaults ) {
	$args = array(
		'alignment'              => 'aligncenter',
		'background_color'       => '#000000',
		'background_color_hover' => '#222222',
		'border_radius'          => 4,
		'icon_color'             => '#ffffff',
		'icon_color_hover'       => '#ffffff',
		'size'                   => 40,
		);
		
	$args = wp_parse_args( $args, $defaults );
	return $args;
}

/* Customize the footer copyright text */
/* @WP: Changed code to add Home Inspector Pro Websites branding */
remove_action( 'genesis_footer', 'genesis_do_footer' );
add_action( 'genesis_footer', 'imagely_custom_footer' );
function imagely_custom_footer() {
	?>
	<p>Copyright &copy; <?php echo date('Y'); ?> <a href="<?php echo esc_url( home_url( '/' ) ); ?>" rel="home"><?php bloginfo( 'name' ); ?></a>. All rights reserved. <?php echo ( get_theme_mod( 'powered_by_imagely' ) ) ? 'Powered by <a href="http://homeinspectorsites.com/" rel="nofollow">Home Inspector Pro Websites</a>.' : "" ?></p>
	<?php
}

/**
* @WP: Make Header Shrink on Page Scroll
**/
 
add_action ('wp_footer','vr_shrink_head',1);
function vr_shrink_head() {
?>
 
<script>
jQuery(document).ready(function($) {
    $(window).scroll(function () {
        if ($(window).scrollTop() > 100) { 
            $('header').addClass('shrink');
        }
        else{
            $('header').removeClass('shrink');
        }
    });
});
</script>
 
<?php 
}

//* Remove the edit link
add_filter ( 'genesis_edit_post_link' , '__return_false' );