/**
 * Pro Theme JS file
 *
 * This file contains all theme JS functions.
 */
jQuery(document).ready(function($){

	playActiveVideos();

	function playActiveVideos(){
		setTimeout( function(){
			var video = $( '.swiper-slide-active' ).find( 'video' );

			if( 'undefined' == typeof( video.attr( 'customizer' ) ) )
				$( '.swiper-slide-active' ).find( 'video' ).trigger( 'play' );

		}, 100 );
	}

	$( '.swiper-container' ).each(function(){

		var thisSwiperContainer = $(this);
		var thisSwiper = $( $(this)[0] ).swiper();

		thisSwiper.on( 'touchEnd' , function ( s ) {

			console.log( s );

			thisSwiperContainer.find( 'video' ).trigger( 'pause' );

			playActiveVideos();
		});

	});

	// Move videos in the customizer to their first frame
	$('video').each(function(){

		// Get the video object
		video = $(this);

		// Generate a preview
		if( !video.prop( 'autoplay' ) && video.prop( 'customizer' ) ) {
			// Trigger play
			video.trigger( 'play' );

			// As soon as we play then trigger a stop
			video.on( 'play', function(){
				setTimeout( function(){
					// Trigger pause
					video.trigger( 'pause' );
				}, 100);
			});
		};

	});

	/**
	 * Tabs
	 */
	$('.layers-pro-tabs').each(function( index, element ){

		var $widget              = $( element );
		var $tab_buttons_holder  = $widget.find('.nav-tabs');
		var $tab_buttons         = $tab_buttons_holder.find('li');
		var $tab_contents_holder = $widget.find('.tab-content');
		var $tab_contents        = $tab_contents_holder.children();

		$tab_buttons.each(function( index, element ){

			$( element ).on( 'click', function() {

				var $related_tab_button = $(this);
				var $related_tab_content = $tab_contents.eq(index);

				$tab_buttons.not($related_tab_button).removeClass('active');
				$related_tab_button.addClass('active');

				$tab_contents.not($related_tab_content).hide();
				$related_tab_content.show();

				return false;
			});
		});
	});

	/**
	 * Accordion
	 */
	$('.layers-pro-accordion').each(function( index, element ){

		var $widget              = $( element );
		var $accordion_list      = $widget.find('.accordion-list');
		var $accordion_items     = $widget.find('.accordion-list li');

		$accordion_items.each(function( index, element ){

			$( element ).children('section').slideUp(0).css({ position: 'relative', visibility: 'visible' });

			$( element ).children('a').on( 'click', function() {

				var $related_accordion_title   = $(this);
				var $related_accordion_item    = $related_accordion_title.parent();
				var $related_accordion_content = $related_accordion_item.children('section');

				if ( $related_accordion_item.hasClass('active') ) {
					// Close
					$related_accordion_item.removeClass('active');
					$related_accordion_content.slideUp({ duration:200, easing: 'layersEaseInOut' });
				}
				else {
					// Open
					$related_accordion_item.addClass('active');
					$related_accordion_content.slideDown({ duration:200, easing: 'layersEaseInOut' });
				}

				return false;
			});
		});
	});

	/**
	 * Search
	 */
	$(document).on( 'click', '.search-close', function(){

		$('.search-interface-overlay')
			.removeClass('search-interface-play-in')
			.addClass('search-interface-play-out');

		return false;

	});
	$(document).on( 'click', '.header-search > a, .menui-item-layers-search > a', function(){

		var $search_field = $('#layers-modal-search-field');

		$('.search-interface-overlay')
			.removeClass('search-interface-play-out')
			.addClass('search-interface-play-in');

		setTimeout(function() {
			$search_field
				.focus()
				.layers_pro_set_cursor_postion( $search_field.val().length );
		}, 600 );

		return false;
	});

});

/**
 * Helper function that initialsizes our Carousel. It expects that there
 * is a row of our standard Layers post list HTML that it will convert
 *
 * @param  string widget_id         Widgets id selector eg #widget_66
 * @param  object pass_settings     Settings object
 * @return Swiper                   Swiper object
 */
function layers_pro_init_post_carousel( widget_id, pass_settings ) {

	// Create a holder to hold the cloned elements that we will subsequently mimic in our carousel.
	jQuery( widget_id ).prepend('<div class="layers-pro-post-carousel-mimic-row" >');

	// Get this element.
	var mimic_row = jQuery( widget_id ).find('.layers-pro-post-carousel-mimic-row');

	// Hide this mimic row until it's needed - it will be shown, DOM measurments taken from it, then hidden again before it's rendered.
	mimic_row.hide();

	// Copy the classes from the row so the CSS (especially responsive) make it behave in the same way.
	mimic_row.addClass( jQuery( widget_id ).children('.row').attr('class') );

	// Clone one of the article elements that we will mimic (clone many during debugging stage).
	mimic_row.prepend( jQuery( widget_id ).children('.row').children(':lt(6)').clone() );

	// Empty the articles so they don't tax the browsers performance.
	mimic_row.children().html('').prepend('<div style="background:black">&nbsp;</div>');

	// Now use the mimic row to take measurement from - the mimic row will get it's measurements from our CSS.
	var right_margin = layers_pro_post_carousel_calculate_right_margin( widget_id + ' .layers-pro-post-carousel-mimic-row' );
	var per_row = layers_pro_post_carousel_calculate_per_row( widget_id + ' .layers-pro-post-carousel-mimic-row' );

	// Get the all important row that will will contain our carousel.
	var carousel_row = jQuery( widget_id ).children('.row').not('.layers-pro-post-carousel-mimic-row');

	// Wrap the elemnts in the necessary holders required for Swiper.
	carousel_row.children().not('.swiper-pagination, .arrows').wrapAll('<div class="swiper-wrapper">');
	carousel_row.children().wrapAll('<div class="swiper-container">');

	// Get the final element that will become our carousel.
	carousel_element = carousel_row.find('.swiper-container');

	// Setup the slides/columns.
	carousel_element.find('.column, .column-flush')
		// Remove framewrok classes fromt the column that will disrupt the carousel display.
		.removeClass('column column-flush span-1 span-2 span-3 span-4 span-5 span-6 span-7 span-8 span-9 span-10 span-11 span-12')
		// Add the swiper-slide class that Swiper needs.
		.addClass('swiper-slide');

	// Set the defaults for the accordion.
	defaults = {
		mode                :'horizontal',
		calculateHeight     : true,
		wrapperClass        : 'swiper-wrapper',
		slideClass          : 'swiper-slide',
		spaceBetween        : right_margin,
		slidesPerView       : per_row,
		loop                : true,
		paginationClickable : true,
	};

	// Merge defaults with the settings passed.
	settings = jQuery.extend( defaults, pass_settings );

	// Init the Carousel.
	var post_swiper = carousel_element.swiper( settings );

	/*
	post_swiper.enableKeyboardControl();

	jQuery( widget_id ).find('.arrows a').on( 'click' , function(e){
		e.preventDefault();

		// "Hi Mom"
		$that = jQuery(this);

		if( $that.hasClass( 'swiper-pagination-switch' ) ){ // Anchors
			post_swiper.swipeTo( $that.index() );
		} else if( $that.hasClass( 'l-left-arrow' ) ){ // Previous
			post_swiper.swipePrev();
		} else if( $that.hasClass( 'l-right-arrow' ) ){ // Next
			post_swiper.swipeNext();
		}

		return false;
	});
	*/

	// Setup resize event so the carousel is constantly mimicing our mimic row - and auto heighting it's columns.
	jQuery(window).resize(function(){

		var right_margin = layers_pro_post_carousel_calculate_right_margin( widget_id + ' .layers-pro-post-carousel-mimic-row' );
		var per_row = layers_pro_post_carousel_calculate_per_row( widget_id + ' .layers-pro-post-carousel-mimic-row' );

		post_swiper.params.spaceBetween = right_margin;
		post_swiper.params.slidesPerView = per_row;
		post_swiper.update();

		// Also auto height match the rows.
		layers_pro_carousel_swiper_resize( post_swiper );
	});

	layers_pro_carousel_swiper_resize( post_swiper );

	// Return the slider incase it is needed further.
	return post_swiper;
}

/**
 * Helper function that looks at a row and culculates how many columns
 * it is displaying per row, as a result of the CSS that is applied to
 * it. Epecially in the case of responsive CSS. This is important for
 * Used so the Carousel can mimic the frameworks column behaviour.
 *
 * @param  string $row_element_selector   Selector that will help us find the row to mimic
 * @return number
 */
function layers_pro_post_carousel_calculate_per_row( $row_element_selector ){

	// Cache the necessary elements.
	var row_element = jQuery( $row_element_selector );
	var column_element = row_element.find('.column, .column-flush').eq(0);

	// Flash show the element so it holds size in the DOM
	row_element.show();

	// Calculate how many columns there are per_row.
	var per_row = parseInt( Math.round( row_element.width() / column_element.width() ) );

	// Hide the element soon as the calculations are done.
	row_element.hide();

	return per_row;
}

/**
 * Helper function, similar to above, looks at a row and calculates the
 * right margin at any given time, so we can mimic the column spacing
 * in the Carousel too.
 *
 * @param  string $row_element_selector   Selector that will help us find the row to mimic
 * @return number                         Right margin pixel width (without the px)
 */
function layers_pro_post_carousel_calculate_right_margin( $row_element_selector ){

	// Cache the necessary elements.
	var row_element = jQuery( $row_element_selector );
	var column_element = row_element.find('.column, .column-flush').eq(0);

	// Flash show the element so it holds size in the DOM
	row_element.show();

	// Calculate the margin-right.
	var right_margin = column_element.css('margin-right');
	right_margin = ( undefined !== right_margin ) ? parseInt( right_margin.replace( 'px', '' ) ) : 0 ;

	// Hide the element soon as the calculations are done.
	row_element.hide();

	return right_margin;
}

/**
 * Helper function that calculates the highest column and makes sure
 * all other rows match it's height.
 *
 * @param  object s Swiper Object
 */
function layers_pro_carousel_swiper_resize( s ){

	// Reset the container and thumbnail-body height, if it has been set on the previous resize iteration.
	// Transition must be set to 0s so the postions can be measured immediately and not after animations
    s.container.css({ height: '' });
    s.container.find('.thumbnail-body').css( 'height', '' );
    s.container.find('.thumbnail-body .button').css( { 'margin-top': '', transition: '0s,0s' } );

    // Don't do any height matching if there is only one slide showing - this happens on Mobile.
    if ( s.params.slidesPerView <= 1 ) return;

    // Find the highest slide, and set the Swiper container to that height.
    var height = 0;
	s.slides.each(function( key, slide ){
    	var slide_height = jQuery(slide).outerHeight();
        if( height < slide_height ) height = slide_height;
    });
    s.container.css({ height: height+'px' });

    // Then go through each slide and stretch the thumbnail-body to fill in the empty space.
    s.slides.each( function( key, slide ) {

		// Skip auto-heighting of 'Display: Overlay' blocks.
    	if ( jQuery(slide).hasClass('with-overlay') ) return true;

        var thumb_height = 0;
        var padding_top = 0;
        var padding_bottom = 0;
        var slide_height = jQuery(slide).height();

        if ( jQuery(slide).find( '.thumbnail-media' ).length ) {
        	thumb_height = jQuery(slide).find( '.thumbnail-media' ).outerHeight();
        }

        if ( jQuery(slide).find( '.thumbnail-body' ).length ) {
        	padding_top = jQuery(slide).find( '.thumbnail-body' ).css( 'padding-top' );
			padding_bottom = jQuery(slide).find( '.thumbnail-body' ).css( 'padding-top' );

			var $body_height = parseInt(slide_height) - parseInt(thumb_height) - parseInt(padding_top) - parseInt(padding_bottom)

			jQuery(slide).find( '.thumbnail-body' ).height( $body_height );

			// If there's a button then also 'marging-top' exact amount
			// so that all buttons align along the bottom of the blocks.
			if ( jQuery(slide).find( '.thumbnail-body .button' ).length ) {

				var $content_height = jQuery(slide).find( '.thumbnail-body' ).children().eq(0).height();

				var $button = jQuery(slide).find( '.thumbnail-body .button' );
				$button_margin_top = parseInt( $button.css('margin-top') );
				$button_margin_bottom = parseInt( $button.css('margin-bottom') );

				$new_margin = parseInt( $body_height - $content_height + $button_margin_top + $button_margin_bottom );

				$button.css( { 'margin-top': $new_margin } );
				$button.css( { transition: '' } );
			}
        }
    });

}

// Set cursor position
jQuery.fn.layers_pro_set_cursor_postion = function(pos) {
	this.each(function(index, elem) {
		if (elem.setSelectionRange) {
			elem.setSelectionRange(pos, pos);
		}
		else if (elem.createTextRange) {
			var range = elem.createTextRange();
			range.collapse(true);
			range.moveEnd('character', pos);
			range.moveStart('character', pos);
			range.select();
		}
	});
	return this;
};
