<?php
/**
 * Carousel Widget
 *
 * This file is used to register and display the Layers - Carousel widget.
 *
 * @package Layers
 * @since Layers 1.0.0
 */

function layers_call_to_action_widget_init() {

	if( !class_exists( 'Layers_Call_To_Action_Widget' ) ) {
		class Layers_Call_To_Action_Widget extends Layers_Widget {

			/**
			*  Widget construction
			*/
			function Layers_Call_To_Action_Widget(){

				/**
				* Widget variables
				*
			 	* @param  	string    		$widget_title    	Widget title
			 	* @param  	string    		$widget_id    		Widget slug for use as an ID/classname
			 	* @param  	array 			$checkboxes    	(optional) Array of checkbox names to be saved in this widget. Don't forget these please!
			 	*/
				$this->widget_title = __( 'Call to Action', 'layers-pro' );
				$this->widget_id    = 'layers-pro-call-to-action';
				$this->checkboxes   = array(
					'buttons-full-width',
				);

				/* Widget settings. */
				$widget_ops = array(
					'classname'   => 'obox-layers-' . $this->widget_id .'-widget',
					'description' => __( 'This widget is used to display your ', 'layers-pro' ) . $this->widget_title . '.',
				);

				/* Widget control settings. */
				$control_ops = array(
					'width'   => LAYERS_WIDGET_WIDTH_LARGE,
					'height'  => NULL,
					'id_base' => LAYERS_THEME_SLUG . '-widget-' . $this->widget_id,
				);

				/* Create the widget. */
				parent::__construct(
					LAYERS_THEME_SLUG . '-widget-' . $this->widget_id,
					$this->widget_title,
					$widget_ops,
					$control_ops
				);

				/* Setup Widget Defaults */
				$this->defaults = array (
					'title' => __( 'Call to Action', 'layers-pro' ),
					'excerpt' => __( 'Lorem ipsum dolor sit amet enim ad minim veniam, quis nostrud exercitation.', 'layers-pro' ),
					'design' => array(
						'layout' => 'layout-boxed',
						'buttons-size' => 'medium',
						'background' => array(
							'position' => 'center',
							'repeat' => 'no-repeat',
							'color' => NULL,
						),
						'fonts' => array(
							'align' => 'text-left',
							'size' => 'medium',
							'color' => NULL,
							'shadow' => NULL
						),
					),
				);

				/* Setup Widget Repeater Defaults */
				$this->register_repeater_defaults(
					'button',
					2,
					array(
						'link' => '#',
						'link_text' => 'Buy Now',
						'buttons-size' => 'medium',
					),
					array(
						'link' => '#',
						'link_text' => 'More Info',
						'buttons-size' => 'medium',
					)
				);
			}

			/**
			*  Widget front end display
			*/
			function widget( $args, $instance ) {

				// Turn $args array into variables.
				extract( $args );

				// $instance Defaults
				$instance_defaults = $this->defaults;

				// If we have information in this widget instance, then ignore the defaults.
				if( empty( $instance ) ) {
					$instance = wp_parse_args( $instance, $instance_defaults );
				}

				// Easier to understand
				$widget = $instance;

				/**
				 * Styling
				 */

				// Widget Styling
				if ( ! empty( $widget['design']['background'] ) ) {
					layers_inline_styles( "#{$widget_id}", 'background', array( 'background' => $widget['design']['background'] ) );
				}

				// Apply the advanced widget styling
				$this->apply_widget_advanced_styling( $widget_id, $widget );

				/**
				 * Generate Classes
				 */
				$classes = array();
				$classes[] = 'widget';
				$classes[] = 'layers-pro-call-to-action';
				$classes[] = 'content-vertical-massive';
				$classes[] = $this->check_and_return( $widget , 'design', 'advanced', 'customclass' ); // Apply custom class from design-bar's advanced control.
				$classes = implode( ' ', apply_filters( 'layers_content_widget_container_class', $classes ) );
				?>
				<section id="<?php echo $widget_id; ?>" class="<?php echo esc_attr( $classes ); ?>">

					<?php if ( NULL !== $this->check_and_return( $widget , 'title' ) || NULL !== $this->check_and_return( $widget , 'excerpt' ) ) { ?>
						<div class="container clearfix">
							<?php
							/**
							 * Generate Classes
							 */
							$classes = array();
							$classes[] = $this->check_and_return( $widget , 'design', 'fonts', 'size' );
							$classes[] = $this->check_and_return( $widget , 'design', 'fonts', 'align' );
							$classes[] = ( $this->check_and_return( $widget, 'design', 'background', 'color' ) && 'dark' == layers_is_light_or_dark( $this->check_and_return( $widget, 'design', 'background', 'color' ) ) ? 'invert' : '' );
							$classes = implode( ' ', $classes );
							?>
							<div class="section-title clearfix <?php echo esc_attr( $classes ); ?>">
								<?php if( '' != $widget['title'] ) { ?>
									<h3 class="heading"><?php echo esc_html( $widget['title'] ); ?></h3>
								<?php } ?>
								<?php if( '' != $widget['excerpt'] ) { ?>
									<div class="excerpt"><?php layers_the_content( $widget['excerpt'] ); ?></div>
								<?php } ?>
							</div>
						</div>
					<?php } ?>

					<?php if( ! empty( $widget['buttons'] ) ) { ?>

						<?php
						/**
						 * Generate Classes
						 */
						$classes = array();
						$classes[] = 'row';
						$classes[] = 'button-collection';
						$classes[] = $this->get_widget_layout_class( $widget );
						$classes[] = $this->check_and_return( $widget , 'design', 'fonts', 'align' );
						$classes[] = $this->check_and_return( $widget , 'design', 'liststyle' );
						$classes = implode( ' ', $classes );
						?>
						<div class="<?php echo esc_attr( $classes ); ?>">
							<?php foreach ( explode( ',', $widget['button_ids'] ) as $item_key ) {

								// Setup Internal Vars.
								$item             = $widget['buttons'][ $item_key ];
								$item_id_attr     = "{$widget_id}-buttons-{$item_key}";

								// Make sure we've got a tab going on here
								if( ! isset( $widget['buttons'][$item_key ]) ) continue;

								// Make sure we've got minimum stuff required for a button.
								if ( ! $this->check_and_return( $item, 'link' ) || ! $this->check_and_return( $item, 'link_text' ) ) continue;

								/**
								 * Button Styling
								 */
								layers_pro_apply_widget_button_styling( $this, $item, array( "#{$widget_id} #{$item_id_attr}.button" ) );

								// Hover styling (to be removed).
								if ( ! empty( $item['design']['buttons-background-color'] ) ) {

									$background       = $item['design']['buttons-background-color'];
									$background_hover = layers_too_light_then_dark( $background, 20 );
									$color            = layers_get_light_or_dark( $background, '#454545', '#ffffff' ); // Light, Dark.

									// Apply button hover
									layers_inline_styles( "#{$widget_id} #{$item_id_attr}.button:hover", array( 'css' => array(
										'background'    => $background_hover,
									) ) );
								}

								/**
								 * Generate Classes
								 */
								$classes = array();
								// Button Size
								if ( $this->check_and_return( $item, 'design', 'buttons-size' ) ) {
									$classes[] = 'btn-' . $this->check_and_return( $item, 'design', 'buttons-size' );
								}
								// Button Width
								if ( $this->check_and_return( $item, 'design', 'buttons-full-width' ) ) {
									$classes[] = 'btn-full';
								}
								$classes = implode( ' ', $classes );
								?>
								<a
									href="<?php echo $item['link']; ?>"
									id="<?php echo esc_attr( $item_id_attr ); ?>"
									class="button <?php echo esc_attr( $classes ); ?>"
									<?php echo ( 'new' == $this->check_and_return( $item, 'design', 'buttons-target' ) ) ? 'target="_blank"' : ''; ?>
									>
									<?php echo $item['link_text']; ?>
								</a>
							<?php } ?>
						</div>

					<?php } ?>

				</section>
			<?php }

			/**
			*  Widget update
			*/
			function update( $new_instance, $old_instance ) {
				if ( isset( $this->checkboxes ) ) {
					foreach( $this->checkboxes as $cb ) {
						if( isset( $old_instance[ $cb ] ) ) {
							$old_instance[ $cb ] = strip_tags( $new_instance[ $cb ] );
						}
					} // foreach checkboxes
				} // if checkboxes
				return $new_instance;
			}

			/**
			* Widget form
			*
			* We use regular HTML here, it makes reading the widget much easier than if we used just php to echo all the HTML out.
			*/
			function form( $instance ){

				// $instance Defaults
				$instance_defaults = $this->defaults;

				// If we have information in this widget instance, then ignore the defaults.
				if( empty( $instance ) ) {
					$instance = wp_parse_args( $instance, $instance_defaults );
				}

				// Set the widget vars.
				$widget = $instance;

				$this->design_bar(
					'side', // CSS Class Name
					array( // Widget Object
						'name' => $this->get_layers_field_name( 'design' ),
						'id' => $this->get_layers_field_id( 'design' ),
					),
					$widget, // Widget Values
					array( // Components
						'layout',
						'fonts',
						'background',
						'advanced',
					)
				); ?>
				<div class="layers-container-large" id="layers-column-widget-<?php echo $this->number; ?>">

					<?php $this->form_elements()->header( array(
						'title' =>'Content',
						'icon_class' =>'text'
					) ); ?>

					<section class="layers-accordion-section layers-content">

						<p class="layers-form-item">
							<?php echo $this->form_elements()->input(
								array(
									'type' => 'text',
									'name' => $this->get_layers_field_name( 'title' ),
									'id' => $this->get_layers_field_id( 'title' ),
									'placeholder' => __( 'Enter title here', 'layers-pro' ),
									'value' => ( isset( $widget['title'] ) ) ? $widget['title'] : NULL ,
									'class' => 'layers-text layers-large'
								)
							); ?>
						</p>

						<p class="layers-form-item">
							<?php echo $this->form_elements()->input(
								array(
									'type' => 'rte',
									'name' => $this->get_layers_field_name( 'excerpt' ),
									'id' => $this->get_layers_field_id( 'excerpt' ),
									'placeholder' =>  __( 'Short Excerpt', 'layers-pro' ),
									'value' => ( isset( $widget['excerpt'] ) ) ? $widget['excerpt'] : NULL ,
									'class' => 'layers-textarea layers-large'
								)
							); ?>
						</p>

						<div class="layers-form-item">
							<?php $this->repeater( 'button', $widget ); ?>
						</div>

					</section>
				</div>

			<?php }

			/**
			*  Widget repeated item display.
			*/
			function button_item( $item_guid, $widget ) {

				// Required - Get the name of this type.
				$type = str_replace( '_item', '', __FUNCTION__ );
				?>
				<li class="layers-accordion-item <?php echo $this->item_count; ?>" data-guid="<?php echo esc_attr( $item_guid ); ?>">

					<a class="layers-accordion-title">
						<span>
							<?php echo ucfirst( $type ); ?>
							<span class="layers-detail">
								<?php echo ( isset( $widget['title'] ) ? ': ' . substr( stripslashes( strip_tags( $widget['title'] ) ), 0 , 50 ) : NULL ); ?>
								<?php echo ( isset( $widget['title'] ) && strlen( $widget['title'] ) > 50 ? '...' : NULL ); ?>
							</span>
						</span>
					</a>

					<section class="layers-accordion-section layers-content">

						<?php $this->design_bar(
							'top', // CSS Class Name
							array( // Widget Object
								'name'       => $this->get_layers_field_name( 'design' ),
								'id'         => $this->get_layers_field_id( 'design' ),
								'number'     => $this->number,
								'show_trash' => FALSE,
							),
							$widget, // Widget Values
							array( // Components
								'buttons-advanced-new' => array(
									'icon-css' => 'icon-call-to-action',
									'label' => __( 'Buttons', 'layers-pro' ),
									'elements' => array(

										'buttons-size' => array(
											'type' => 'select',
											'label' => __( 'Size', 'layers-pro' ),
											'name' => $this->get_layers_field_name( 'design', 'buttons-size' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-size' ),
											'value' => ( isset( $widget['design']['buttons-size'] ) ) ? $widget['design']['buttons-size'] : NULL,
											'options' => array(
												'small' => __( 'Small', 'layers-pro' ),
												'medium' => __( 'Medium', 'layers-pro' ),
												'large' => __( 'Large', 'layers-pro' ),
												'massive' => __( 'Massive', 'layers-pro' ),
											),
										),
										'buttons-full-width' => array(
											'type' => 'checkbox',
											'label' => __( 'Full Width', 'layers-pro' ),
											'name' => $this->get_layers_field_name( 'design', 'buttons-full-width' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-full-width' ),
											'value' => ( isset( $widget['design']['buttons-full-width'] ) ) ? $widget['design']['buttons-full-width'] : NULL,
										),
										'buttons-target' => array(
											'type' => 'select',
											'label' => __( 'Link Target', 'layers-pro' ),
											'name' => $this->get_layers_field_name( 'design', 'buttons-target' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-target' ),
											'value' => ( isset( $widget['design']['buttons-target'] ) ) ? $widget['design']['buttons-target'] : NULL,
											'options' => array(
												'this' => __( 'This tab', 'layers-pro' ),
												'new' => __( 'New tab', 'layers-pro' ),
											),
										),

										/**
										 * Background
										 */
										'buttons-background-style' => array(
											'type'  => 'select',
											'label' => __( 'Background Style', 'layers-pro' ),
											'class' => 'group layers-push-top-small',
											'choices' => array(
												'solid'       => __( 'Solid', 'layers-pro' ),
												'transparent' => __( 'Transparent', 'layers-pro' ),
												'gradient'    => __( 'Gradient', 'layers-pro' ),
											),
											'default'  => 'solid',

											'name' => $this->get_layers_field_name( 'design', 'buttons-background-style' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-background-style' ),
											'value' => ( isset( $widget['design']['buttons-background-style'] ) ) ? $widget['design']['buttons-background-style'] : NULL,
										),
										'buttons-background-color' => array(
											'type' => 'color',
											'label' => __( 'Background Color', 'layers-pro' ),
											'class' => 'group',
											'data' => array(
												'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'buttons-background-style' ),
												'show-if-value' => 'solid',
											),
											'name' => $this->get_layers_field_name( 'design', 'buttons-background-color' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-background-color' ),
											'value' => ( isset( $widget['design']['buttons-background-color'] ) ) ? $widget['design']['buttons-background-color'] : NULL,
										),
										'buttons-background-gradient-start-color' => array(
											'type' => 'color',
											'label' => __( 'Gradient Start Color', 'layers-pro' ),
											'class' => 'group',
											'data' => array(
												'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'buttons-background-style' ),
												'show-if-value' => 'gradient',
											),
											'name' => $this->get_layers_field_name( 'design', 'buttons-background-gradient-start-color' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-background-gradient-start-color' ),
											'value' => ( isset( $widget['design']['buttons-background-gradient-start-color'] ) ) ? $widget['design']['buttons-background-gradient-start-color'] : NULL,
										),
										'buttons-background-gradient-end-color' => array(
											'type' => 'color',
											'label' => __( 'Gradient End Color', 'layers-pro' ),
											'class' => 'group',
											'data' => array(
												'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'buttons-background-style' ),
												'show-if-value' => 'gradient',
											),
											'name' => $this->get_layers_field_name( 'design', 'buttons-background-gradient-end-color' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-background-gradient-end-color' ),
											'value' => ( isset( $widget['design']['buttons-background-gradient-end-color'] ) ) ? $widget['design']['buttons-background-gradient-end-color'] : NULL,
										),
										'buttons-background-gradient-direction' => array(
											'type' => 'range',
											'label' => __( 'Gradient Angle', 'layers-pro' ),
											'class' => 'group',
											'min' => '0',
											'max' => '360',
											'step' => '1',
											'data' => array(
												'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'buttons-background-style' ),
												'show-if-value' => 'gradient',
											),
											'name' => $this->get_layers_field_name( 'design', 'buttons-background-gradient-direction' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-background-gradient-direction' ),
											'value' => ( isset( $widget['design']['buttons-background-gradient-direction'] ) ) ? $widget['design']['buttons-background-gradient-direction'] : NULL,
										),
										/**
										 * Text
										 */
										'buttons-text-color' => array(
											'type' => 'color',
											'label' => __( 'Text Color', 'layers-pro' ),
											'class' => 'group layers-push-top-small',

											'name' => $this->get_layers_field_name( 'design', 'buttons-text-color' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-text-color' ),
											'value' => ( isset( $widget['design']['buttons-text-color'] ) ) ? $widget['design']['buttons-text-color'] : NULL,
										),
										'buttons-text-shadow' => array(
											'type' => 'select',
											'label' => __( 'Text Shadow', 'layers-pro' ),
											'class' => 'group',
											'choices' => array(
												'none' => __( 'None', 'layers-pro' ),
												'bottom' => __( 'Bottom Shadow', 'layers-pro' ),
												'top' => __( 'Top Shadow', 'layers-pro' ),
											),
											'name' => $this->get_layers_field_name( 'design', 'buttons-text-shadow' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-text-shadow' ),
											'value' => ( isset( $widget['design']['buttons-text-shadow'] ) ) ? $widget['design']['buttons-text-shadow'] : NULL,
										),
										'buttons-text-transform' => array(
											'type' => 'select',
											'label' => __( 'Text Transform', 'layers-pro' ),
											'class' => 'group',
											'choices' => array(
												'normal' => __( 'Normal', 'layers-pro' ),
												'uppercase' => __( 'Uppercase', 'layers-pro' ),
												'capitalize' => __( 'Capitalize', 'layers-pro' ),
												'lowercase' => __( 'Lowercase', 'layers-pro' ),
											),
											'name' => $this->get_layers_field_name( 'design', 'buttons-text-transform' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-text-transform' ),
											'value' => ( isset( $widget['design']['buttons-text-transform'] ) ) ? $widget['design']['buttons-text-transform'] : NULL,
										),
										/**
										 * Border
										 */
										'buttons-border-width' => array(
											'type' => 'range',
											'label' => __( 'Border Width', 'layers-pro' ),
											'class' => 'group layers-push-top-small',
											'min' => '0',
											'max' => '10',
											'step' => '1',
											'default' => '0',
											'name' => $this->get_layers_field_name( 'design', 'buttons-border-width' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-border-width' ),
											'value' => ( isset( $widget['design']['buttons-border-width'] ) ) ? $widget['design']['buttons-border-width'] : NULL,
										),
										'buttons-border-color' => array(
											'type' => 'color',
											'label' => __( 'Border Color', 'layers-pro' ),
											'class' => 'group',
											'name' => $this->get_layers_field_name( 'design', 'buttons-border-color' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-border-color' ),
											'value' => ( isset( $widget['design']['buttons-border-color'] ) ) ? $widget['design']['buttons-border-color'] : NULL,
										),
										/**
										 * Styling
										 */
										'buttons-border-radius' => array(
											'type' => 'range',
											'label' => __( 'Rounded Corner Size', 'layers-pro' ),
											'class' => 'group layers-push-top-small',
											'min' => '0',
											'max' => '100',
											'step' => '1',
											'default' => '4',
											'name' => $this->get_layers_field_name( 'design', 'buttons-border-radius' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-border-radius' ),
											'value' => ( isset( $widget['design']['buttons-border-radius'] ) ) ? $widget['design']['buttons-border-radius'] : NULL,
										),
										'buttons-shadow' => array(
											'type' => 'select',
											'label' => __( 'Button Shadow', 'layers-pro' ),
											'class' => 'group',
											'choices' => array(
												'none' => __( 'None', 'layers-pro' ),
												'small' => __( 'Small', 'layers-pro' ),
												'medium' => __( 'Medium', 'layers-pro' ),
												'large' => __( 'Large', 'layers-pro' ),
											),
											'name' => $this->get_layers_field_name( 'design', 'buttons-shadow' ),
											'id' => $this->get_layers_field_id( 'design', 'buttons-shadow' ),
											'value' => ( isset( $widget['design']['buttons-shadow'] ) ) ? $widget['design']['buttons-shadow'] : NULL,
										),

									),
								),
							)
						); ?>
						<div class="layers-row">

							<div class="layers-row">
								<p class="layers-form-item layers-column layers-span-6">
									<label for="<?php echo $this->get_layers_field_id( 'link' ); ?>">
										<?php _e( 'Button Link', 'layers-pro' ); ?>
									</label>
									<?php echo $this->form_elements()->input(
										array(
											'type' => 'text',
											'name' => $this->get_layers_field_name( 'link' ),
											'id' => $this->get_layers_field_id( 'link' ),
											'placeholder' => __( 'http://', 'layers-pro' ),
											'value' => ( isset( $widget['link'] ) ) ? $widget['link'] : NULL ,
											'class' => 'layers-text',
										)
									); ?>
								</p>

								<p class="layers-form-item layers-column layers-span-6">
									<label for="<?php echo $this->get_layers_field_id( 'link_text' ); ?>">
										<?php _e( 'Button Text', 'layers-pro' ); ?>
									</label>
									<?php echo $this->form_elements()->input(
										array(
											'type' => 'text',
											'name' => $this->get_layers_field_name( 'link_text' ),
											'id' => $this->get_layers_field_id( 'link_text' ),
											'placeholder' => __( 'e.g. "Read More"', 'layers-pro' ),
											'value' => ( isset( $widget['link_text'] ) ) ? $widget['link_text'] : NULL ,
										)
									); ?>
								</p>

							</div>
						</div>
					</section>

				</li>
				<?php
			}

		}

		// Add our function to the widgets_init hook.
		 register_widget("Layers_Call_To_Action_Widget");
	}
}
add_action( 'widgets_init', 'layers_call_to_action_widget_init', 40 );
