<?php
/**
 * Carousel Widget
 *
 * This file is used to register and display the Layers - Carousel widget.
 *
 * @package Layers
 * @since Layers 1.0.0
 */

function layers_carousel_widget_init() {

	if( !class_exists( 'Layers_Carousel_Widget' ) ) {
		class Layers_Carousel_Widget extends Layers_Widget {

			/**
			*  Widget construction
			*/
			function Layers_Carousel_Widget(){

				/**
				* Widget variables
				*
				* @param  	string    		$widget_title    	Widget title
				* @param  	string    		$widget_id    		Widget slug for use as an ID/classname
				* @param  	string    		$post_type    		(optional) Post type for use in widget options
				* @param  	string    		$taxonomy    		(optional) Taxonomy slug for use as an ID/classname
				* @param  	array 			$checkboxes    	(optional) Array of checkbox names to be saved in this widget. Don't forget these please!
				*/
				$this->widget_title = __( 'Posts Carousel' , 'layers-pro' );
				$this->widget_id = 'layers-pro-post-carousel';
				$this->post_type = 'post';
				$this->taxonomy = 'category';
				$this->checkboxes = array(
					'show_media',
					'show_titles',
					'show_excerpts',
					'show_dates',
					'show_author',
					'show_tags',
					'show_categories',
					'show_call_to_action',
					'navigation_arrows',
					'navigation_dots',
					'autoplay_slides',
				); // @TODO: Try make this more dynamic, or leave a different note reminding users to change this if they add/remove checkboxes

				/* Widget settings. */
				$widget_ops = array(
					'classname'   => 'obox-layers-' . $this->widget_id .'-widget',
					'description' => __( 'This widget is used to display your ', 'layers-pro' ) . $this->widget_title . '.',
				);

				/* Widget control settings. */
				$control_ops = array(
					'width'   => LAYERS_WIDGET_WIDTH_SMALL,
					'height'  => NULL,
					'id_base' => LAYERS_THEME_SLUG . '-widget-' . $this->widget_id,
				);

				/* Create the widget. */
				parent::__construct(
					LAYERS_THEME_SLUG . '-widget-' . $this->widget_id ,
					$this->widget_title,
					$widget_ops,
					$control_ops
				);

				/* Setup Widget Defaults */
				$this->defaults = array(
					'title' => __( 'Latest Posts', 'layers-pro' ),
					'excerpt' => __( 'Stay up to date with all our latest news and launches. Only the best quality makes it onto our blog!', 'layers-pro' ),
					'category' => 0,
					'posts_per_page' => 6,
					'design' => array(
						
						'slider_type' => 'set-by-set',
						'navigation_arrows' => 'on',
						'navigation_arrows_color' => '#696969',
						'navigation_dots' => 'on',
						'autoplay_slides' => 'on',
						'slide_interval' => '4',

						'show_media' => 'on',
						'show_titles' => 'on',
						'show_excerpts' => 'on',
						'show_dates' => 'on',
						'show_author' => 'on',
						'show_tags' => 'on',
						'show_categories' => 'on',
						'excerpt_length' => 200,
						'show_call_to_action' => 'on',
						'call_to_action' => __( 'Read More' , 'layers-pro' ),

						'text_style' => 'regular',

						'layout' => 'layout-boxed',
						'imageratios' => 'image-square',
						'textalign' => 'text-left',
						'columns' => '3',
						'gutter' => 'on',
						
						'buttons-size' => 'medium',
						
						'background' => array(
							'position' => 'center',
							'repeat' => 'no-repeat'
						),
						'fonts' => array(
							'align' => 'text-left',
							'size' => 'medium',
							'color' => NULL,
							'shadow' => NULL
						),
					),
				);
			}

			/**
			* Enqueue Scripts
			*/
			function enqueue_scripts(){

				// Slider JS enqueue
				wp_enqueue_script(
					LAYERS_THEME_SLUG . '-slider-js' ,
					get_template_directory_uri() . '/core/widgets/js/swiper.js',
					array( 'jquery' ),
					LAYERS_VERSION
				);

				// Slider CSS enqueue
				wp_enqueue_style(
					LAYERS_THEME_SLUG . '-slider',
					get_template_directory_uri() . '/core/widgets/css/swiper.css',
					array(),
					LAYERS_VERSION
				);
			}

			/**
			*  Widget front end display
			*/
			function widget( $args, $instance ) {

				// Turn $args array into variables.
				extract( $args );

				// $instance Defaults
				$instance_defaults = $this->defaults;

				// If we have information in this widget, then ignore the defaults
				if( !empty( $instance ) ) $instance_defaults = array();

				// Parse $instance
				$widget = wp_parse_args( $instance, $instance_defaults );

				// Enqueue Scipts when needed
				$this->enqueue_scripts();

				// Set the span class for each column
				if( isset( $widget['design'][ 'columns']  ) ) {
					$col_count = str_ireplace('columns-', '', $widget['design'][ 'columns']  );
					$span_class = 'span-' . ( 12/ $col_count );
				} else {
					$col_count = 3;
					$span_class = 'span-4';
				}

				/**
				 * Styling
				 */

				// Widget Styling
				if( NULL !== $this->check_and_return( $widget, 'design', 'background' ) ) layers_inline_styles( "#{$widget_id}", 'background', array( 'background' => $this->check_and_return( $widget, 'design', 'background' ) ) );
				if( NULL !== $this->check_and_return( $widget, 'design', 'fonts', 'color' ) ) layers_inline_styles( "#{$widget_id}", 'color', array( 'selectors' => array( '.section-title h3.heading' , '.section-title div.excerpt' ) , 'color' => $this->check_and_return( $widget, 'design', 'fonts', 'color' ) ) );
				if( NULL !== $this->check_and_return( $widget, 'design', 'column-background-color' ) ) layers_inline_styles( "#{$widget_id} .layers-masonry-column", array( 'css' => array( 'background-color' => $this->check_and_return( $widget, 'design', 'column-background-color' ) ) ) );
				if( NULL !== $this->check_and_return( $widget, 'design', 'column-background-color' ) ) layers_inline_styles( "#{$widget_id} .thumbnail-body", array( 'css' => array( 'background-color' => 'transparent' ) ) );
				if( NULL !== $this->check_and_return( $widget, 'design', 'navigation_arrows_color' ) ) layers_inline_styles( "#{$widget_id}", 'color', array( 'selectors' => array( '.arrows a' ), 'color' => $this->check_and_return( $widget, 'design', 'navigation_arrows_color' ) ) );
				if( NULL !== $this->check_and_return( $widget, 'design', 'navigation_arrows_color' ) ) layers_inline_styles( "#{$widget_id}", 'border', array( 'selectors' => array( 'span.swiper-pagination-switch' ), 'border' => array( 'color' => $this->check_and_return( $widget, 'design', 'navigation_arrows_color' ) ) ) );
				if( NULL !== $this->check_and_return( $widget, 'design', 'navigation_arrows_color' ) ) layers_inline_styles( "#{$widget_id}", 'background', array( 'selectors' => array( 'span.swiper-pagination-switch' ), 'background' => array( 'color' => $this->check_and_return( $widget, 'design', 'navigation_arrows_color' ) ) ) );
				if( NULL !== $this->check_and_return( $widget, 'design', 'navigation_arrows_color' ) ) layers_inline_styles( "#{$widget_id}", 'background', array( 'selectors' => array( 'span.swiper-pagination-switch.swiper-active-switch' ), 'background' => array( 'color' => 'transparent !important' ) ) );
				
				// Button Styling
				layers_pro_apply_widget_button_styling( $this, $widget, array( "#{$widget_id} .button" ) );

				// Apply the advanced widget styling
				$this->apply_widget_advanced_styling( $widget_id, $widget );

				// Set Image Sizes
				if( isset( $widget['design'][ 'imageratios' ] ) ){

					// Translate Image Ratio
					$image_ratio = layers_translate_image_ratios( $widget['design'][ 'imageratios' ] );

					if( 'layout-boxed' == $this->check_and_return( $widget , 'design', 'layout' ) && $col_count > 2 ){
						$use_image_ratio = $image_ratio . '-medium';
					} elseif( 'layout-boxed' != $this->check_and_return( $widget , 'design', 'layout' ) && $col_count > 3 ){
						$use_image_ratio = $image_ratio . '-large';
					} else {
						$use_image_ratio = $image_ratio . '-large';
					}
				} else {
					$use_image_ratio = 'large';
				}

				$query_args[ 'post_type' ] = $this->post_type;
				$query_args[ 'posts_per_page' ] = $widget['posts_per_page'];
				if( isset( $widget['order'] ) ) {

					$decode_order = json_decode( $widget['order'], true );

					if( is_array( $decode_order ) ) {
						foreach( $decode_order as $key => $value ){
							$query_args[ $key ] = $value;
						}
					}
				}

				// Do the special taxonomy array()
				if( isset( $widget['category'] ) && '' != $widget['category'] && 0 != $widget['category'] ){

					$query_args['tax_query'] = array(
						array(
							"taxonomy" => $this->taxonomy,
							"field" => "id",
							"terms" => $widget['category']
						)
					);
				}

				// Do the WP_Query
				$post_query = new WP_Query( $query_args );

				// Set the meta to display
				global $layers_post_meta_to_display;
				$layers_post_meta_to_display = array();
				if( isset( $widget['show_dates'] ) ) $layers_post_meta_to_display[] = 'date';
				if( isset( $widget['show_author'] ) ) $layers_post_meta_to_display[] = 'author';
				if( isset( $widget['show_categories'] ) ) $layers_post_meta_to_display[] = 'categories';
				if( isset( $widget['show_tags'] ) ) $layers_post_meta_to_display[] = 'tags';

				/**
				 * Generate Classes
				 */
				$classes = array();
				$classes[] = 'widget';
				$classes[] = 'widget-post-carousel';
				$classes[] = 'row';
				$classes[] = 'content-vertical-massive';
				$classes[] = $this->check_and_return( $widget , 'design', 'advanced', 'customclass' ); // Apply custom class from design-bar's advanced control.
				$classes = implode( ' ', apply_filters( 'layers_post_widget_container_class' , $classes ) );
				?>
				<section id="<?php echo $widget_id; ?>" class="<?php echo esc_attr( $classes ); ?>">
					<?php if ( NULL !== $this->check_and_return( $widget , 'title' ) || NULL !== $this->check_and_return( $widget , 'excerpt' ) ) { ?>
						<div class="container clearfix">
							<?php
							/**
							 * Generate Classes
							 */
							$classes = array();
							$classes[] = $this->check_and_return( $widget , 'design', 'fonts', 'size' );
							$classes[] = $this->check_and_return( $widget , 'design', 'fonts', 'align' );
							$classes[] = ( $this->check_and_return( $widget, 'design', 'background' , 'color' ) && 'dark' == layers_is_light_or_dark( $this->check_and_return( $widget, 'design', 'background' , 'color' ) ) ? 'invert' : '' );
							$classes = implode( ' ', $classes ); ?>
							<div class="section-title clearfix <?php echo esc_attr( $classes ); ?>">
								<?php if( '' != $widget['title'] ) { ?>
									<h3 class="heading"><?php echo esc_html( $widget['title'] ); ?></h3>
								<?php } ?>
								<?php if( '' != $widget['excerpt'] ) { ?>
									<div class="excerpt"><?php layers_the_content( $widget['excerpt'] ); ?></div>
								<?php } ?>
							</div>
						</div>
					<?php } ?>

					<div class="row post-carousel-row <?php echo $this->get_widget_layout_class( $widget ); ?>">
						<?php
						if( $post_query->have_posts() ) :
							while( $post_query->have_posts() ) :

								$post_query->the_post();

								/**
								 * Generate Classes
								 */
								$classes = array();
								$classes[] = $span_class;
								$classes[] = 'column' . ( 'on' != $this->check_and_return( $widget, 'design', 'gutter' ) ? '-flush' : '' );
								$classes[] = ( 'overlay' == $this->check_and_return( $widget, 'design', 'text_style' ) && has_post_thumbnail() ) ? 'with-overlay' : 'regular' ;
								$classes[] = ( NULL !== $this->check_and_return( $widget, 'design', 'column-background-color' ) && 'dark' == layers_is_light_or_dark( $this->check_and_return( $widget, 'design', 'column-background-color' ) ) ) ? 'invert' : '' ;
								$classes = implode( ' ' , $classes );
								?>
								<article class="layers-masonry-column thumbnail <?php echo esc_attr( $classes ); ?>" data-cols="<?php echo $col_count; ?>">

									<?php // Layers Featured Media
									if( isset( $widget['design']['show_media'] ) ) {
										echo layers_post_featured_media(
											array(
												'postid' => get_the_ID(),
												'wrap_class' => 'thumbnail-media' .  ( ( isset( $column['design'][ 'imageratios' ] ) && 'image-round' == $column['design'][ 'imageratios' ] ) ? ' image-rounded' : '' ),
												'size' => $use_image_ratio,
												'hide_href' => false
											)
										);
									} ?>
									<?php if( isset( $widget['design']['show_titles'] ) || isset( $widget['design']['show_excerpts'] ) ) { ?>
										<div class="thumbnail-body">
											<div class="overlay">
												<?php if( isset( $widget['design']['show_titles'] ) ) { ?>
													<header class="article-title">
														<h4 class="heading"><a href="<?php the_permalink(); ?>"><?php the_title(); ?></a></h4>
													</header>
												<?php } ?>
												<?php if( isset( $widget['design']['show_excerpts'] ) ) {
													if( isset( $widget['design']['excerpt_length'] ) && '' == $widget['design']['excerpt_length'] ) {
														echo '<div class="excerpt">';
															the_content();
														echo '</div>';
													} else if( isset( $widget['design']['excerpt_length'] ) && 0 != $widget['design']['excerpt_length'] && strlen( get_the_excerpt() ) > $widget['design']['excerpt_length'] ){
														echo '<div class="excerpt">' . substr( get_the_excerpt() , 0 , $widget['design']['excerpt_length'] ) . '&#8230;</div>';
													} else if( '' != get_the_excerpt() ){
														echo '<div class="excerpt">' . get_the_excerpt() . '</div>';
													}
												}; ?>
												<?php if( 'overlay' != $this->check_and_return( $widget, 'design', 'text_style' ) ) { ?>
													<?php layers_post_meta( get_the_ID(), $layers_post_meta_to_display, 'footer' , 'meta-info ' . ( NULL !== $this->check_and_return( $widget, 'design', 'column-background-color' ) && 'dark' == layers_is_light_or_dark( $this->check_and_return( $widget, 'design', 'column-background-color' ) ) ? 'invert' : '' ) );?>
												<?php } // Don't show meta if we have chosen overlay ?>
												<?php if( isset( $widget['design']['show_call_to_action'] ) && $this->check_and_return( $widget, 'design', 'call_to_action' ) ) { ?>

													<?php
													/**
													 * Generate Classes
													 */
													$classes = array();
													// Button Size
													if ( $this->check_and_return( $widget, 'design', 'buttons-size' ) ) {
														$classes[] = 'btn-' . $this->check_and_return( $widget, 'design', 'buttons-size' );
													}
													// Button Width
													if ( $this->check_and_return( $widget, 'design', 'buttons-full-width' ) ) {
														$classes[] = 'btn-full';
													}
													$classes = implode( ' ', $classes );
													?>
													<a
														href="<?php the_permalink(); ?>"
														class="button <?php echo esc_attr( $classes ); ?>"
														<?php echo ( 'new' == $this->check_and_return( $widget, 'design', 'buttons-target' ) ) ? 'target="_blank"' : ''; ?>
														>
														<?php echo $widget['design']['call_to_action']; ?>
													</a>

												<?php } ?>
											</div>
										</div>
									<?php } ?>

								</article>
								<?php
							endwhile;
						endif;
						?>

						<?php if( isset( $widget['design']['navigation_dots'] ) ) { ?>
							<div class="swiper-pagination"></div>
						<?php } ?>

						<?php if( isset( $widget['design']['navigation_arrows'] ) ) { ?>
							<div class="arrows">
								<a href="" class="l-left-arrow animate"></a>
								<a href="" class="l-right-arrow animate"></a>
							</div>
						<?php } ?>

					</div>
				</section>

				<?php
				/**
				 * Slider javascript initialize
				 */
				$post_swiper_js_obj = str_replace( '-' , '_' , $this->get_layers_field_id( 'slider' ) );
				?>
				<script>
				jQuery(function($){

					<?php echo $post_swiper_js_obj; ?> = layers_pro_init_post_carousel( '#<?php echo $widget_id; ?>', {
						bulletClass: 'swiper-pagination-switch',
						bulletActiveClass: 'swiper-active-switch swiper-visible-switch',

						<?php if( isset( $widget['design']['navigation_dots'] ) ) { ?>
							pagination : '#<?php echo $widget_id; ?> .swiper-pagination',
						<?php } ?>

						<?php if( isset( $widget['design']['navigation_arrows'] ) ) { ?>
							prevButton: '#<?php echo $widget_id; ?> .l-left-arrow',
							nextButton: '#<?php echo $widget_id; ?> .l-right-arrow',
						<?php } ?>

						<?php if( isset( $widget['design']['autoplay_slides'] ) ) { ?>
							autoplay : <?php echo ( isset( $widget['design']['slide_interval'] ) ) ? ( 1000 * $widget['design']['slide_interval'] ) : 3000 ; ?>,
						<?php } ?>

						<?php if( isset( $widget['design']['slider_type'] ) && 'one-by-one' == $widget['design']['slider_type'] ) { ?>
							slidesPerGroup: 1,
						<?php } ?>
					});
				})
				</script>

				<?php
				// Reset WP_Query
				wp_reset_postdata();
				?>

			<?php }

			/**
			*  Widget update
			*/

			function update( $new_instance, $old_instance ) {

				if ( isset( $this->checkboxes ) ) {
					foreach( $this->checkboxes as $cb ) {
						if( isset( $old_instance[ $cb ] ) ) {
							$old_instance[ $cb ] = strip_tags( $new_instance[ $cb ] );
						}
					} // foreach checkboxes
				} // if checkboxes
				return $new_instance;
			}

			/**
			*  Widget form
			*
			* We use regulage HTML here, it makes reading the widget much easier than if we used just php to echo all the HTML out.
			*
			*/
			function form( $instance ){

				// $instance Defaults
				$instance_defaults = $this->defaults;

				// If we have information in this widget, then ignore the defaults
				if( !empty( $instance ) ) $instance_defaults = array();

				// Parse $instance
				$widget = wp_parse_args( $instance, $instance_defaults );

				$this->widget = $widget;


				$this->design_bar(
					'side', // CSS Class Name
					array(
						'name' => $this->get_layers_field_name( 'design' ),
						'id' => $this->get_layers_field_id( 'design' ),
					), // Widget Object
					$widget, // Widget Values
					array( // Components
						'layout',
						'columns',
						'carousel' => array(
							'icon-css' => 'icon-slider',
							'label' => __( 'Carousel', 'layers-pro' ),
							'elements' => array(
								/*
								'slider_type' => array(
									'type' => 'select',
									'name' => $this->get_layers_field_name( 'design', 'slider_type' ),
									'id' => $this->get_layers_field_id( 'design', 'slider_type' ),
									'value' => ( isset( $widget['design']['slider_type'] ) ) ? $widget['design']['slider_type'] : NULL,
									'label' => __( 'Slider Type' , 'layers-pro' ),
									'options' => array(
										'one-by-one' => __( 'Slide one by one' , 'layers-pro' ),
										'set-by-set' => __( 'Slide set by set' , 'layers-pro' ),
									),
								),
								*/
								'navigation_arrows' => array(
									'type' => 'checkbox',
									'name' => $this->get_layers_field_name( 'design', 'navigation_arrows' ),
									'id' => $this->get_layers_field_id( 'design', 'navigation_arrows' ),
									'value' => ( isset( $widget['design']['navigation_arrows'] ) ) ? $widget['design']['navigation_arrows'] : NULL,
									'label' => __( 'Show Slider Arrows' , 'layers-pro' ),
								),
								'navigation_arrows_color' => array(
									'type' => 'color',
									'name' => $this->get_layers_field_name( 'design', 'navigation_arrows_color' ),
									'id' => $this->get_layers_field_id( 'design', 'navigation_arrows_color' ),
									'value' => ( isset( $widget['design']['navigation_arrows_color'] ) ) ? $widget['design']['navigation_arrows_color'] : NULL,
									'label' => __( 'Slider Controls Color' , 'layers-pro' ),
									'data' => array( 'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'navigation_arrows' ), 'show-if-value' => 'true' ),
								),
								'navigation_dots' => array(
									'type' => 'checkbox',
									'name' => $this->get_layers_field_name( 'design', 'navigation_dots' ),
									'id' => $this->get_layers_field_id( 'design', 'navigation_dots' ),
									'value' => ( isset( $widget['design']['navigation_dots'] ) ) ? $widget['design']['navigation_dots'] : NULL,
									'label' => __( 'Show Slider Dots' , 'layers-pro' ),
								),
								'autoplay_slides' => array(
									'type' => 'checkbox',
									'name' => $this->get_layers_field_name( 'design', 'autoplay_slides' ),
									'id' => $this->get_layers_field_id( 'design', 'autoplay_slides' ),
									'value' => ( isset( $widget['design']['autoplay_slides'] ) ) ? $widget['design']['autoplay_slides'] : NULL,
									'label' => __( 'Autoplay Slides' , 'layers-pro' ),
								),
								'slide_interval' => array(
									'type' => 'number',
									'name' => $this->get_layers_field_name( 'design', 'slide_interval' ),
									'id' => $this->get_layers_field_id( 'design', 'slide_interval' ),
									'value' => ( isset( $widget['design']['slide_interval'] ) ) ? $widget['design']['slide_interval'] : NULL,
									'label' => __( 'Slide Interval (seconds)' , 'layers-pro' ),
									'data' => array( 'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'autoplay_slides' ), 'show-if-value' => 'true' ),
								),
							),
						),
						'display' => array(
							'icon-css' => 'icon-display',
							'label' => __( 'Display', 'layers-pro' ),
							'elements' => array(
								'text_style' => array(
									'type' => 'select',
									'name' => $this->get_layers_field_name( 'design', 'text_style' ),
									'description' => __( 'All posts must have featured-images', 'layers-pro' ),
									'id' => $this->get_layers_field_id( 'design', 'text_style' ),
									'value' => ( isset( $widget['design']['text_style'] ) ) ? $widget['design']['text_style'] : NULL,
									'label' => __( 'Title &amp; Excerpt Position' , 'layers-pro' ),
									'options' => array(
											'regular' => __( 'Regular' , 'layers-pro' ),
											'overlay' => __( 'Overlay' , 'layers-pro' )
									),
								),
								'show_media' => array(
									'type' => 'checkbox',
									'name' => $this->get_layers_field_name( 'design', 'show_media' ),
									'id' => $this->get_layers_field_id( 'design', 'show_media' ),
									'value' => ( isset( $widget['design']['show_media'] ) ) ? $widget['design']['show_media'] : NULL,
									'label' => __( 'Show Featured Images' , 'layers-pro' ),
								),
								'show_titles' => array(
									'type' => 'checkbox',
									'name' => $this->get_layers_field_name( 'design', 'show_titles' ),
									'id' => $this->get_layers_field_id( 'design', 'show_titles' ),
									'value' => ( isset( $widget['design']['show_titles'] ) ) ? $widget['design']['show_titles'] : NULL,
									'label' => __( 'Show  Post Titles' , 'layers-pro' ),
								),
								'show_excerpts' => array(
									'type' => 'checkbox',
									'name' => $this->get_layers_field_name( 'design', 'show_excerpts' ),
									'id' => $this->get_layers_field_id( 'design', 'show_excerpts' ),
									'value' => ( isset( $widget['design']['show_excerpts'] ) ) ? $widget['design']['show_excerpts'] : NULL,
									'label' => __( 'Show Post Excerpts' , 'layers-pro' ),
								),
								'excerpt_length' => array(
									'type' => 'number',
									'name' => $this->get_layers_field_name( 'design', 'excerpt_length' ),
									'id' => $this->get_layers_field_id( 'design', 'excerpt_length' ),
									'min' => 0,
									'max' => 10000,
									'value' => ( isset( $widget['design']['excerpt_length'] ) ) ? $widget['design']['excerpt_length'] : NULL,
									'label' => __( 'Excerpts Length' , 'layers-pro' ),
									'data' => array( 'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'show_excerpts' ), 'show-if-value' => 'true' ),
								),
								'show_dates' => array(
									'type' => 'checkbox',
									'name' => $this->get_layers_field_name( 'design', 'show_dates' ),
									'id' => $this->get_layers_field_id( 'design', 'show_dates' ),
									'value' => ( isset( $widget['design']['show_dates'] ) ) ? $widget['design']['show_dates'] : NULL,
									'label' => __( 'Show Post Dates' , 'layers-pro' ),
								),
								'show_author' => array(
									'type' => 'checkbox',
									'name' => $this->get_layers_field_name( 'design', 'show_author' ),
									'id' => $this->get_layers_field_id( 'design', 'show_author' ),
									'value' => ( isset( $widget['design']['show_author'] ) ) ? $widget['design']['show_author'] : NULL,
									'label' => __( 'Show Post Author' , 'layers-pro' ),
								),
								'show_tags' => array(
									'type' => 'checkbox',
									'name' => $this->get_layers_field_name( 'design', 'show_tags' ),
									'id' => $this->get_layers_field_id( 'design', 'show_tags' ),
									'value' => ( isset( $widget['design']['show_tags'] ) ) ? $widget['design']['show_tags'] : NULL,
									'label' => __( 'Show Tags' , 'layers-pro' ),
								),
								'show_categories' => array(
									'type' => 'checkbox',
									'name' => $this->get_layers_field_name( 'design', 'show_categories' ),
									'id' => $this->get_layers_field_id( 'design', 'show_categories' ),
									'value' => ( isset( $widget['design']['show_categories'] ) ) ? $widget['design']['show_categories'] : NULL,
									'label' => __( 'Show Categories' , 'layers-pro' ),
								),
								'show_call_to_action' => array(
									'type' => 'checkbox',
									'name' => $this->get_layers_field_name( 'design', 'show_call_to_action' ),
									'id' => $this->get_layers_field_id( 'design', 'show_call_to_action' ),
									'value' => ( isset( $widget['design']['show_call_to_action'] ) ) ? $widget['design']['show_call_to_action'] : NULL,
									'label' => __( 'Show "Read More" Buttons' , 'layers-pro' ),
								),
								'call_to_action' => array(
									'type' => 'text',
									'name' => $this->get_layers_field_name( 'design', 'call_to_action' ),
									'id' => $this->get_layers_field_id( 'design', 'call_to_action' ),
									'value' => ( isset( $widget['design']['call_to_action'] ) ) ? $widget['design']['call_to_action'] : NULL,
									'label' => __( '"Read More" Text' , 'layers-pro' ),
									'data' => array( 'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'show_call_to_action' ), 'show-if-value' => 'true' ),
								),
							),
						),
						'fonts',
						'buttons-advanced-new' => array(
							'icon-css' => 'icon-call-to-action',
							'label' => __( 'Buttons', 'layers-pro' ),
							'elements' => array(

								'buttons-size' => array(
									'type' => 'select',
									'label' => __( 'Size', 'layers-pro' ),
									'name' => $this->get_layers_field_name( 'design', 'buttons-size' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-size' ),
									'value' => ( isset( $widget['design']['buttons-size'] ) ) ? $widget['design']['buttons-size'] : NULL,
									'options' => array(
										'small' => __( 'Small', 'layers-pro' ),
										'medium' => __( 'Medium', 'layers-pro' ),
										'large' => __( 'Large', 'layers-pro' ),
										'massive' => __( 'Massive', 'layers-pro' ),
									),
								),
								'buttons-full-width' => array(
									'type' => 'checkbox',
									'label' => __( 'Full Width', 'layers-pro' ),
									'name' => $this->get_layers_field_name( 'design', 'buttons-full-width' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-full-width' ),
									'value' => ( isset( $widget['design']['buttons-full-width'] ) ) ? $widget['design']['buttons-full-width'] : NULL,
								),
								'buttons-target' => array(
									'type' => 'select',
									'label' => __( 'Link Target', 'layers-pro' ),
									'name' => $this->get_layers_field_name( 'design', 'buttons-target' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-target' ),
									'value' => ( isset( $widget['design']['buttons-target'] ) ) ? $widget['design']['buttons-target'] : NULL,
									'options' => array(
										'this' => __( 'This tab', 'layers-pro' ),
										'new' => __( 'New tab', 'layers-pro' ),
									),
								),
								
								/**
								 * Background
								 */
								'buttons-background-style' => array(
									'type'  => 'select',
									'label' => __( 'Background Style', 'layers-pro' ),
									'class' => 'group layers-push-top-small',
									'choices' => array(
										'solid'       => __( 'Solid', 'layers-pro' ),
										'transparent' => __( 'Transparent', 'layers-pro' ),
										'gradient'    => __( 'Gradient', 'layers-pro' ),
									),
									'default'  => 'solid',
									
									'name' => $this->get_layers_field_name( 'design', 'buttons-background-style' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-background-style' ),
									'value' => ( isset( $widget['design']['buttons-background-style'] ) ) ? $widget['design']['buttons-background-style'] : NULL,
								),
								'buttons-background-color' => array(
									'type' => 'color',
									'label' => __( 'Background Color', 'layers-pro' ),
									'class' => 'group',
									'data' => array(
										'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'buttons-background-style' ),
										'show-if-value' => 'solid',
									),
									'name' => $this->get_layers_field_name( 'design', 'buttons-background-color' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-background-color' ),
									'value' => ( isset( $widget['design']['buttons-background-color'] ) ) ? $widget['design']['buttons-background-color'] : NULL,
								),
								'buttons-background-gradient-start-color' => array(
									'type' => 'color',
									'label' => __( 'Gradient Start Color', 'layers-pro' ),
									'class' => 'group',
									'data' => array(
										'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'buttons-background-style' ),
										'show-if-value' => 'gradient',
									),
									'name' => $this->get_layers_field_name( 'design', 'buttons-background-gradient-start-color' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-background-gradient-start-color' ),
									'value' => ( isset( $widget['design']['buttons-background-gradient-start-color'] ) ) ? $widget['design']['buttons-background-gradient-start-color'] : NULL,
								),
								'buttons-background-gradient-end-color' => array(
									'type' => 'color',
									'label' => __( 'Gradient End Color', 'layers-pro' ),
									'class' => 'group',
									'data' => array(
										'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'buttons-background-style' ),
										'show-if-value' => 'gradient',
									),
									'name' => $this->get_layers_field_name( 'design', 'buttons-background-gradient-end-color' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-background-gradient-end-color' ),
									'value' => ( isset( $widget['design']['buttons-background-gradient-end-color'] ) ) ? $widget['design']['buttons-background-gradient-end-color'] : NULL,
								),
								'buttons-background-gradient-direction' => array(
									'type' => 'range',
									'label' => __( 'Gradient Angle', 'layers-pro' ),
									'class' => 'group',
									'min' => '0',
									'max' => '360',
									'step' => '1',
									'data' => array(
										'show-if-selector' => '#' . $this->get_layers_field_id( 'design', 'buttons-background-style' ),
										'show-if-value' => 'gradient',
									),
									'name' => $this->get_layers_field_name( 'design', 'buttons-background-gradient-direction' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-background-gradient-direction' ),
									'value' => ( isset( $widget['design']['buttons-background-gradient-direction'] ) ) ? $widget['design']['buttons-background-gradient-direction'] : NULL,
								),
								/**
								 * Text
								 */
								'buttons-text-color' => array(
									'type' => 'color',
									'label' => __( 'Text Color', 'layers-pro' ),
									'class' => 'group layers-push-top-small',
									
									'name' => $this->get_layers_field_name( 'design', 'buttons-text-color' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-text-color' ),
									'value' => ( isset( $widget['design']['buttons-text-color'] ) ) ? $widget['design']['buttons-text-color'] : NULL,
								),
								'buttons-text-shadow' => array(
									'type' => 'select',
									'label' => __( 'Text Shadow', 'layers-pro' ),
									'class' => 'group',
									'choices' => array(
										'none' => __( 'None', 'layers-pro' ),
										'bottom' => __( 'Bottom Shadow', 'layers-pro' ),
										'top' => __( 'Top Shadow', 'layers-pro' ),
									),
									'name' => $this->get_layers_field_name( 'design', 'buttons-text-shadow' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-text-shadow' ),
									'value' => ( isset( $widget['design']['buttons-text-shadow'] ) ) ? $widget['design']['buttons-text-shadow'] : NULL,
								),
								'buttons-text-transform' => array(
									'type' => 'select',
									'label' => __( 'Text Transform', 'layers-pro' ),
									'class' => 'group',
									'choices' => array(
										'normal' => __( 'Normal', 'layers-pro' ),
										'uppercase' => __( 'Uppercase', 'layers-pro' ),
										'capitalize' => __( 'Capitalize', 'layers-pro' ),
										'lowercase' => __( 'Lowercase', 'layers-pro' ),
									),
									'name' => $this->get_layers_field_name( 'design', 'buttons-text-transform' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-text-transform' ),
									'value' => ( isset( $widget['design']['buttons-text-transform'] ) ) ? $widget['design']['buttons-text-transform'] : NULL,
								),
								/**
								 * Border
								 */
								'buttons-border-width' => array(
									'type' => 'range',
									'label' => __( 'Border Width', 'layers-pro' ),
									'class' => 'group layers-push-top-small',
									'min' => '0',
									'max' => '10',
									'step' => '1',
									'default' => '0',
									'name' => $this->get_layers_field_name( 'design', 'buttons-border-width' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-border-width' ),
									'value' => ( isset( $widget['design']['buttons-border-width'] ) ) ? $widget['design']['buttons-border-width'] : NULL,
								),
								'buttons-border-color' => array(
									'type' => 'color',
									'label' => __( 'Border Color', 'layers-pro' ),
									'class' => 'group',
									'name' => $this->get_layers_field_name( 'design', 'buttons-border-color' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-border-color' ),
									'value' => ( isset( $widget['design']['buttons-border-color'] ) ) ? $widget['design']['buttons-border-color'] : NULL,
								),
								/**
								 * Styling
								 */
								'buttons-border-radius' => array(
									'type' => 'range',
									'label' => __( 'Rounded Corner Size', 'layers-pro' ),
									'class' => 'group layers-push-top-small',
									'min' => '0',
									'max' => '100',
									'step' => '1',
									'default' => '4',
									'name' => $this->get_layers_field_name( 'design', 'buttons-border-radius' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-border-radius' ),
									'value' => ( isset( $widget['design']['buttons-border-radius'] ) ) ? $widget['design']['buttons-border-radius'] : NULL,
								),
								'buttons-shadow' => array(
									'type' => 'select',
									'label' => __( 'Button Shadow', 'layers-pro' ),
									'class' => 'group',
									'choices' => array(
										'none' => __( 'None', 'layers-pro' ),
										'small' => __( 'Small', 'layers-pro' ),
										'medium' => __( 'Medium', 'layers-pro' ),
										'large' => __( 'Large', 'layers-pro' ),
									),
									'name' => $this->get_layers_field_name( 'design', 'buttons-shadow' ),
									'id' => $this->get_layers_field_id( 'design', 'buttons-shadow' ),
									'value' => ( isset( $widget['design']['buttons-shadow'] ) ) ? $widget['design']['buttons-shadow'] : NULL,
								),

							),
						),
						'imageratios',
						'background',
						'advanced',

					)
				); ?>
				<div class="layers-container-large">

					<?php $this->form_elements()->header( array(
						'title' =>  __( 'Post' , 'layers-pro' ),
						'icon_class' =>'post'
					) ); ?>

					<section class="layers-accordion-section layers-content">

						<div class="layers-row layers-push-bottom">
							<p class="layers-form-item">
								<?php echo $this->form_elements()->input(
									array(
										'type' => 'text',
										'name' => $this->get_layers_field_name( 'title' ),
										'id' => $this->get_layers_field_id( 'title' ),
										'placeholder' => __( 'Enter title here' , 'layers-pro' ),
										'value' => ( isset( $widget['title'] ) ) ? $widget['title'] : NULL,
										'class' => 'layers-text layers-large',
									)
								); ?>
							</p>

							<p class="layers-form-item">
								<?php echo $this->form_elements()->input(
									array(
										'type' => 'rte',
										'name' => $this->get_layers_field_name( 'excerpt' ),
										'id' => $this->get_layers_field_id( 'excerpt' ),
										'placeholder' => __( 'Short Excerpt' , 'layers-pro' ),
										'value' => ( isset( $widget['excerpt'] ) ) ? $widget['excerpt'] : NULL,
										'class' => 'layers-textarea layers-large',
									)
								); ?>
							</p>
							<?php // Grab the terms as an array and loop 'em to generate the $options for the input
							$terms = get_terms( $this->taxonomy , array( 'hide_empty' => false ) );
							if( !is_wp_error( $terms ) ) { ?>
								<p class="layers-form-item">
									<label for="<?php echo $this->get_layers_field_id( 'category' ); ?>"><?php echo __( 'Category to Display' , 'layers-pro' ); ?></label>
									<?php $category_options[ 0 ] = __( 'All' , 'layers-pro' );
									foreach ( $terms as $t ) $category_options[ $t->term_id ] = $t->name;
									echo $this->form_elements()->input(
										array(
											'type' => 'select',
											'name' => $this->get_layers_field_name( 'category' ),
											'id' => $this->get_layers_field_id( 'category' ),
											'placeholder' => __( 'Select a Category' , 'layers-pro' ),
											'value' => ( isset( $widget['category'] ) ) ? $widget['category'] : NULL,
											'options' => $category_options,
										)
									); ?>
								</p>
							<?php } // if !is_wp_error ?>
							<p class="layers-form-item">
								<label for="<?php echo $this->get_layers_field_id( 'posts_per_page' ); ?>"><?php echo __( 'Number of items to show' , 'layers-pro' ); ?></label>
								<?php $select_options[ '-1' ] = __( 'Show All' , 'layers-pro' );
								$select_options = $this->form_elements()->get_incremental_options( $select_options , 1 , 20 , 1);
								echo $this->form_elements()->input(
									array(
										'type' => 'number',
										'name' => $this->get_layers_field_name( 'posts_per_page' ),
										'id' => $this->get_layers_field_id( 'posts_per_page' ),
										'value' => ( isset( $widget['posts_per_page'] ) ) ? $widget['posts_per_page'] : NULL,
										'min' => '-1',
										'max' => '100',
									)
								); ?>
							</p>

							<p class="layers-form-item">
								<label for="<?php echo $this->get_layers_field_id( 'order' ); ?>"><?php echo __( 'Sort by' , 'layers-pro' ); ?></label>
								<?php echo $this->form_elements()->input(
									array(
										'type' => 'select',
										'name' => $this->get_layers_field_name( 'order' ),
										'id' => $this->get_layers_field_id( 'order' ),
										'value' => ( isset( $widget['order'] ) ) ? $widget['order'] : NULL,
										'options' => $this->form_elements()->get_sort_options(),
									)
								); ?>
							</p>
						</div>
					</section>

				</div>
			<?php } // Form
		} // Class

	}

	// Add our function to the widgets_init hook.
	register_widget( 'Layers_Carousel_Widget' );
}
add_action( 'widgets_init', 'layers_carousel_widget_init', 40 );
