<?php
/**
 * Carousel Widget
 *
 * This file is used to register and display the Layers - Carousel widget.
 *
 * @package Layers
 * @since Layers 1.0.0
 */

function layers_tabs_widget_init() {

	if( !class_exists( 'Layers_Tabs_Widget' ) ) {
		class Layers_Tabs_Widget extends Layers_Widget {

			/**
			*  Widget construction
			*/
			function Layers_Tabs_Widget(){

				/**
				* Widget variables
				*
				* @param   string   $widget_title   Widget title
				* @param   string   $widget_id      Widget slug for use as an ID/classname
				* @param   array    $checkboxes     (optional) Array of checkbox names to be saved in this widget. Don't forget these please!
				*/
				$this->widget_title = __( 'Tabs', 'layers-pro' );
				$this->widget_id    = 'layers-pro-tabs';
				$this->checkboxes   = array(
					'buttons-full-width',
				);

				/* Widget settings. */
				$widget_ops = array(
					'classname'   => 'obox-layers-' . $this->widget_id .'-widget',
					'description' => __( 'This widget is used to display your ', 'layers-pro' ) . $this->widget_title . '.',
				);

				/* Widget control settings. */
				$control_ops = array(
					'width'   => LAYERS_WIDGET_WIDTH_LARGE,
					'height'  => NULL,
					'id_base' => LAYERS_THEME_SLUG . '-widget-' . $this->widget_id,
				);

				/* Create the widget. */
				parent::__construct(
					LAYERS_THEME_SLUG . '-widget-' . $this->widget_id,
					$this->widget_title,
					$widget_ops,
					$control_ops
				);

				/* Setup Widget Defaults */
				$this->defaults = array (
					'title' => __( 'Tabs', 'layers-pro' ),
					'excerpt' => __( 'Lorem ipsum dolor sit amet enim ad minim veniam, quis nostrud exercitation.', 'layers-pro' ),
					'design' => array(
						'layout' => 'layout-boxed',
						'buttons-size' => 'medium',
						'fonts' => array(
							'align' => 'text-left',
							'size' => 'medium',
							'color' => NULL,
							'shadow' => NULL
						),
					),
				);

				/* Setup Widget Repeater Defaults */
				$this->register_repeater_defaults(
					'tab',
					3,
					array(
						'tab-title' => __( 'Tab 1', 'layers-pro'),
						'content' => __( '<p>Sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Lorem ipsum dolor sit amet, consectetur adipiscing elit, ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in.</p>', 'layers-pro'),
					),
					array(
						'tab-title' => __( 'Tab 2', 'layers-pro'),
						'content' => __( '<p>Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in. Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.</p>', 'layers-pro'),
					),
					array(
						'tab-title' => __( 'Tab 3', 'layers-pro'),
						'content' => __( '<p>Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in.</p>', 'layers-pro'),
					)
				);
			}

			/**
			*  Widget front end display
			*/
			function widget( $args, $instance ) {

				// Turn $args array into variables.
				extract( $args );

				// Use defaults if $instance is empty.
				if( empty( $instance ) && ! empty( $this->defaults ) ) {
					$instance = wp_parse_args( $instance, $this->defaults );
				}

				// Easier to understand
				$widget = $instance;

				/**
				 * Styling
				 */
				// Widget
				if ( $this->check_and_return( $widget, 'design', 'background' ) ) {
					layers_inline_styles( "#{$widget_id}", 'background', array( 'background' => $widget['design']['background'] ) );
					layers_inline_styles( "#{widget_id} .nav-tabs", 'background', array( 'background' => $widget['design']['background'] ) );
				}
				// Tabs
				if ( $this->check_and_return( $widget, 'design', 'tabs-background-color' ) ) {

					// Prep the colors
					$tab_bg_color = $widget['design']['tabs-background-color'];
					$tab_text_color = layers_get_light_or_dark( $tab_bg_color, 'black', 'white' );

					$tab_hover_bg_color = layers_too_light_then_dark( $tab_bg_color, 40 );
					$tab_hover_text_color = $tab_text_color;

					// Apply the colors to the Tab Buttons.
					layers_inline_styles( array(
						'selectors' => "#{$widget_id} .tab-row li a",
						'css' => array(
							'background-color' => $tab_hover_bg_color,
							'color'            => $tab_text_color,
						),
					));
					layers_inline_styles( array(
						'selectors' => "#{$widget_id} .tab-row li.active a",
						'css' => array(
							'background-color' => $tab_bg_color,
							'color'            => $tab_text_color,
						),
					));

					// Apply the colors to the Tab Content.
					layers_inline_styles( array(
						'selectors' => "#{$widget_id} .tab-content",
						'css' => array(
							'color' => $tab_hover_text_color,
						),
					));
					layers_inline_styles( array(
						'selectors' => "#{$widget_id} .tab-content > .row",
						'css' => array(
							'background-color' => $tab_bg_color,
						),
					));
				}

				// Apply the advanced widget styling
				$this->apply_widget_advanced_styling( $widget_id, $widget );

				/**
				 * Generate Classes
				 */
				$classes = array();
				$classes[] = 'widget';
				$classes[] = 'layers-pro-tabs'; // Important - Widget Class
				$classes[] = 'content-vertical-massive';
				$classes[] = $this->check_and_return( $widget , 'design', 'advanced', 'customclass' ); // Apply custom class from design-bar's advanced control.
				$classes = implode( ' ', apply_filters( 'layers_content_widget_container_class', $classes ) );
				?>
				<section id="<?php echo esc_html( $widget_id ); ?>" class="<?php echo esc_attr( $classes ); ?>">

					<?php if( NULL !== $this->check_and_return( $widget , 'title' ) || NULL !== $this->check_and_return( $widget , 'excerpt' ) ) { ?>
						<div class="container clearfix">
							<?php
							/**
							 * Generate Classes
							 */
							$classes = array();
							$classes[] = $this->check_and_return( $widget , 'design', 'fonts', 'size' );
							$classes[] = $this->check_and_return( $widget , 'design', 'fonts', 'align' );
							$classes[] = ( $this->check_and_return( $widget, 'design', 'background' , 'color' ) && 'dark' == layers_is_light_or_dark( $this->check_and_return( $widget, 'design', 'background' , 'color' ) ) ? 'invert' : '' );
							$classes = implode( ' ', $classes ); ?>
							<div class="section-title clearfix <?php echo esc_attr( $classes ); ?>">
								<?php if( '' != $widget['title'] ) { ?>
									<h3 class="heading"><?php echo esc_html( $widget['title'] ); ?></h3>
								<?php } ?>
								<?php if( '' != $widget['excerpt'] ) { ?>
									<div class="excerpt"><?php layers_the_content( $widget['excerpt'] ); ?></div>
								<?php } ?>
							</div>
						</div>
					<?php } ?>

					<?php if( ! empty( $widget['tabs'] ) ) { ?>

						<?php
						/**
						 * Generate Classes
						 */
						$classes = array();
						$classes[] = ( $this->check_and_return( $widget, 'design', 'tabs-background-color' ) ) ? 'no-background' : '';
						$classes[] = $this->get_widget_layout_class( $widget );
						$classes = implode( ' ', $classes );
						?>
						<div class="row tab-row nav-tabs no-inset <?php echo esc_attr( $classes ); ?>">
							<ul>
								<?php
								/**
								 * Loop Items
								 */
								$i = 0;
								foreach ( explode( ',', $widget['tab_ids'] ) as $item_key ) {

									// Setup Internal Vars.
									$item             = $widget['tabs'][ $item_key ];
									$item_id_attr     = "{$widget_id}-tabs-{$item_key}";

									// Make sure we've got a tab going on here
									if( ! isset( $widget['tabs'][$item_key ]) ) continue;

									if( ! $this->check_and_return( $item, 'tab-title' ) ) continue;

									// Calculate Buttons Size.
									$tab_size = 'medium';
									if ( $this->check_and_return( $widget, 'design', 'tabs-size' ) ) {
										$tab_size = $this->check_and_return( $widget, 'design', 'tabs-size' );
									}

									/**
									 * Generate Classes
									 */
									$classes = array();
									$classes[] = 'tab';
									if( 0 == $i )
										$classes[] = 'active';

									if( $this->check_and_return( $widget, 'design', 'tabs-background-color' ) && 'dark' == layers_is_light_or_dark( $this->check_and_return( $widget, 'design', 'tabs-background-color' ) ) ) {
										$classes[] = 'invert';
									} elseif( $this->check_and_return( $widget, 'design', 'background' , 'color' ) && 'dark' == layers_is_light_or_dark( $this->check_and_return( $widget, 'design', 'background' , 'color' ) ) ) {
										$classes[] = 'invert';
									}

									$classes[] = 'btn-' . $tab_size; // Button Size
									// Button Width
									if ( $this->check_and_return( $item, 'design', 'tabs-full-width' ) ) {
										$classes[] = 'btn-full';
									}
									else if ( $this->check_and_return( $widget, 'design', 'tabs-full-width' ) ) {
										$classes[] = 'btn-full';
									}

									$classes = implode( ' ', $classes );
									?>
									<li
										id="<?php echo esc_attr( $item_id_attr ); ?>"
										class="<?php echo esc_attr( $classes ); ?>"
										>
										<a href="#"><?php echo $this->check_and_return( $item, 'tab-title' ); ?></a>
									</li>
									<?php
									$i++;
								}
								?>
							</ul>
						</div>

						<div class="tab-content">
							<?php
							/**
							 * Loop Items
							 */
							foreach ( explode( ',', $widget['tab_ids'] ) as $item_key ) {

								// Setup Internal Vars.
								$type             = 'tab';
								$item             = $widget['tabs'][ $item_key ];
								$item_id_attr     = "{$widget_id}-tabs-{$item_key}";

								// Make sure we've got a tab going on here
								if( ! isset( $widget['tabs'][$item_key ]) ) continue;


								if( ! $this->check_and_return( $item, 'tab-title' )  || ! $this->check_and_return( $item, 'content' )  ) continue;
								/**
								 * Generate Classes
								 */
								$classes = array();
								$classes[] = $this->check_and_return( $item , 'design', 'fonts', 'align' );
								$classes[] = $this->check_and_return( $item , 'design', 'liststyle' );

								if( $this->check_and_return( $widget, 'design', 'tabs-background-color' ) ) {
									if( 'dark' == layers_is_light_or_dark( $this->check_and_return( $widget, 'design', 'tabs-background-color' ) ) ) {
										$classes[] = 'invert';
									}
								} elseif( $this->check_and_return( $widget, 'design', 'background' , 'color' ) ) {
									if( 'dark' == layers_is_light_or_dark( $this->check_and_return( $widget, 'design', 'background' , 'color' ) ) ) {
										$classes[] = 'invert';
									}
								}

								$classes = implode( ' ', $classes );
								?>
								<div
									id="<?php echo esc_attr( $item_id_attr ); ?>"
									class="row container content <?php echo esc_attr( $classes ); ?>"
									>
									<div class="copy">
										<?php echo layers_the_content( $item['content'] ); ?>
									</div>
								</div>
								<?php
							}
							?>
						</div>
					<?php } ?>

				</section>
			<?php }

			/**
			*  Widget update
			*/
			function update( $new_instance, $old_instance ) {
				if ( isset( $this->checkboxes ) ) {
					foreach( $this->checkboxes as $cb ) {
						if( isset( $old_instance[ $cb ] ) ) {
							$old_instance[ $cb ] = strip_tags( $new_instance[ $cb ] );
						}
					} // foreach checkboxes
				} // if checkboxes
				return $new_instance;
			}

			/**
			* Widget form
			*
			* We use regular HTML here, it makes reading the widget much easier than if we used just php to echo all the HTML out.
			*/
			function form( $instance ){

				// $instance Defaults
				$instance_defaults = $this->defaults;

				// If we have information in this widget instance, then ignore the defaults.
				if( empty( $instance ) ) {
					$instance = wp_parse_args( $instance, $instance_defaults );
				}

				// Set the widget vars.
				$widget = $instance;

				$this->design_bar(
					'side', // CSS Class Name
					array( // Widget Object
						'name' => $this->get_layers_field_name( 'design' ),
						'id' => $this->get_layers_field_id( 'design' ),
					),
					$widget, // Widget Values
					array( // Components
						'layout',
						'fonts',
						'background',

						'tab-panel' => array(
							'icon-css' => 'icon-columns',
							'label' => __( 'Tab Panel', 'layers-pro' ),
							'elements' => array(

								'tabs-size' => array(
									'type' => 'select',
									'label' => __( 'Size', 'layers-pro' ),
									'name' => $this->get_layers_field_name( 'design', 'tabs-size' ),
									'id' => $this->get_layers_field_id( 'design', 'tabs-size' ),
									'value' => ( isset( $widget['design']['tabs-size'] ) ) ? $widget['design']['tabs-size'] : NULL,
									'options' => array(
										'small' => __( 'Small', 'layers-pro' ),
										'medium' => __( 'Medium', 'layers-pro' ),
										'large' => __( 'Large', 'layers-pro' )
									),
								),
								'tabs-background-color' => array(
									'type' => 'color',
									'label' => __( 'Background Color', 'layers-pro' ),
									'name' => $this->get_layers_field_name( 'design', 'tabs-background-color' ),
									'id' => $this->get_layers_field_id( 'design', 'tabs-background-color' ),
									'value' => ( isset( $widget['design']['tabs-background-color'] ) ) ? $widget['design']['tabs-background-color'] : NULL,
								),

							),
						),
						'advanced',
					)
				); ?>
				<div class="layers-container-large" id="layers-column-widget-<?php echo $this->number; ?>">

					<?php
					$this->form_elements()->header( array(
						'title' =>'Content',
						'icon_class' =>'text'
					) );
					?>

					<section class="layers-accordion-section layers-content">

						<p class="layers-form-item">
							<?php echo $this->form_elements()->input(
								array(
									'type' => 'text',
									'name' => $this->get_layers_field_name( 'title' ),
									'id' => $this->get_layers_field_id( 'title' ),
									'placeholder' => __( 'Enter title here', 'layers-pro' ),
									'value' => ( isset( $widget['title'] ) ) ? $widget['title'] : NULL,
									'class' => 'layers-text layers-large',
								)
							); ?>
						</p>

						<p class="layers-form-item">
							<?php echo $this->form_elements()->input(
								array(
									'type' => 'rte',
									'name' => $this->get_layers_field_name( 'excerpt' ),
									'id' => $this->get_layers_field_id( 'excerpt' ),
									'placeholder' =>  __( 'Short Excerpt', 'layers-pro' ),
									'value' => ( isset( $widget['excerpt'] ) ) ? $widget['excerpt'] : NULL,
									'class' => 'layers-textarea layers-large',
								)
							); ?>
						</p>

						<div class="layers-form-item">
							<?php $this->repeater( 'tab', $widget ); ?>
						</div>

					</section>

				</div>

			<?php }

			/**
			*  Widget repeated item display.
			*/
			function tab_item( $item_guid, $widget ) {

				// Required - Get the name of this type.
				$type = str_replace( '_item', '', __FUNCTION__ );
				?>
				<li class="layers-accordion-item <?php echo $this->item_count; ?>" data-guid="<?php echo esc_attr( $item_guid ); ?>">

					<a class="layers-accordion-title">
						<span>
							<?php echo ucfirst( $type ); ?>
							<span class="layers-detail">
								<?php echo ( isset( $widget['tab-title'] ) ? ': ' . substr( stripslashes( strip_tags( $widget['tab-title'] ) ), 0 , 50 ) : NULL ); ?>
								<?php echo ( isset( $widget['tab-title'] ) && strlen( $widget['tab-title'] ) > 50 ? '...' : NULL ); ?>
							</span>
						</span>
					</a>

					<section class="layers-accordion-section layers-content">

						<?php
						$this->design_bar(
							'top', // CSS Class Name
							array( // Widget Object
								'name'       => $this->get_layers_field_name( 'design' ),
								'id'         => $this->get_layers_field_id( 'design' ),
								'number'     => $this->number,
								'show_trash' => FALSE,
							),
							$widget, // Widget Values
							array()
						);
						?>

						<div class="layers-row">
							<div class="layers-column layers-span-12">
								<p class="layers-form-item">
									<label for="<?php echo $this->get_layers_field_id( 'tab-title' ); ?>"><?php _e( 'Title' , 'layers-pro' ); ?></label>
									<?php echo $this->form_elements()->input(
										array(
											'type' => 'text',
											'name' => $this->get_layers_field_name( 'tab-title' ),
											'id' => $this->get_layers_field_id( 'tab-title' ),
											'placeholder' => __( 'Tab Title', 'layers-pro' ),
											'value' => ( isset( $widget['tab-title'] ) ) ? $widget['tab-title'] : NULL ,
											'class' => 'layers-text'
										)
									); ?>
								</p>
							</div>
						</div>

						<div class="layers-row">
							<div class="layers-column layers-span-12">
								<p class="layers-form-item">
									<label for="<?php echo $this->get_layers_field_id( 'content' ); ?>"><?php _e( 'Excerpt' , 'layers-pro' ); ?></label>
									<?php echo $this->form_elements()->input(
										array(
											'type' => 'rte',
											'name' => $this->get_layers_field_name( 'content' ),
											'id' => $this->get_layers_field_id( 'content' ),
											'placeholder' =>  __( 'Short Excerpt', 'layers-pro' ),
											'value' => ( isset( $widget['content'] ) ) ? $widget['content'] : NULL ,
											'class' => 'layers-textarea'
										)
									); ?>
								</p>
							</div>
						</div>

					</section>

				</li>
				<?php
			}

		}

		// Add our function to the widgets_init hook.
		register_widget("Layers_Tabs_Widget");
	}
}
add_action( 'widgets_init', 'layers_tabs_widget_init', 40 );
