<?php

/************************************************ 
 * 
 * WP CUSTOMIZER: Register settings and controls 
 * 
 ************************************************/

/* @WP: Removed extra header image description */

add_action( 'customize_register', 'imagely_customizer_register', 16 );
function imagely_customizer_register() {

	global $wp_customize;

	/* Adjust the title and description of the default WordPress "Header Image" section */
	$wp_customize->get_section( 'header_image' )->title = __( 'Logo', 'imagely-free-spirit' );
	$wp_customize->get_section( 'header_image' )->description = __( '', 'imagely-free-spirit' );
	
	/* Remove layout section from Theme Customizer */
        /* @WP: Testing: Restore layout section */
	/* $wp_customize->remove_section( 'genesis_layout' ); */

        /* @WP: Removed Front Featured Slideshow image settings */

	/****************************************************************
	 *
	 * COLORS CUSTOMIZATIONS. We provide extensive color controls via
	 * the customizer. Below we create a new panel, Custom Colors, to 
	 * hold all color controls. Within that panel, we create new 
	 * sections for header, content, footer, and other. And then 
	 * within each section, we provide specific controls for each
	 * relevant element.
	 *  
	 ****************************************************************/
	
	/* Move default background color control to background image section */
	$wp_customize->get_control( 'background_color'  )->section   = 'background_image';
	$wp_customize->get_control( 'background_color'  )->description   = 'The background color will show only if there is no background image.';

	/* Add Customize Colors panel to contain all other colors */
	$wp_customize->add_panel( 'custom-colors', array(
        'priority' => 99,
        'capability' => 'edit_theme_options',
        'theme_supports' => '',
        'title' => __( 'Colors', 'imagely-free-spirit' ),
        'description' => __( 'Customize your website colors.', 'imagely-free-spirit' ),
    ) );

    /* Add Header section to Custom Color Panel */
	$wp_customize->add_section( 'header-colors', array(
        'priority' => 10,
        'capability' => 'edit_theme_options',
        'theme_supports' => '',
        'title' => __( 'Header', 'imagely-free-spirit' ),
        'description' => 'Note: Custom header colors may not apply on your home page if your site has a transparent header area. To see changes, click a different page on the right first.',
        'panel' => 'custom-colors',
    ) );

		/* Header: Background color */
		$wp_customize->add_setting(
			'imagely_header_background',
			array(
				'default'           => '#FbFaFc',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_header_background_control',
				array(
					'description' => __( 'Change the header background color. If your theme has a header background image, this color may not be visible.', 'imagely-free-spirit' ),
				    'label'       => __( 'Header Background', 'imagely-free-spirit' ),
				    'section'     => 'header-colors',
				    'settings'    => 'imagely_header_background',
				)
			)
		);

		/* Header: Title color */
		$wp_customize->add_setting(
			'imagely_title_color',
			array(
				'default'           => '#888',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_title_color_control',
				array(
					'description' => __( 'Change the title color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Title', 'imagely-free-spirit' ),
				    'section'     => 'header-colors',
				    'settings'    => 'imagely_title_color',
				)
			)
		);

		/* Header: Description color */
		$wp_customize->add_setting(
			'imagely_description_color',
			array(
				'default'           => '#888',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_description_color_control',
				array(
					'description' => __( 'Change the description color (if visible).', 'imagely-free-spirit' ),
				    'label'       => __( 'Site Description', 'imagely-free-spirit' ),
				    'section'     => 'header-colors',
				    'settings'    => 'imagely_header_background',
				)
			)
		);

		/* Header: Menu background color */
		$wp_customize->add_setting(
			'imagely_menu_background',
			array(
				'default'           => '#FbFaFc',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_menu_background_control',
				array(
					'description' => __( 'Change the menu background color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Menu Background', 'imagely-free-spirit' ),
				    'section'     => 'header-colors',
				    'settings'    => 'imagely_menu_background',
				)
			)
		);
		
		/* Header: Menu link color */
		$wp_customize->add_setting(
			'imagely_menu_link',
			array(
				'default'           => '#888',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_menu_link_control',
				array(
					'description' => __( 'Change the menu link color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Menu Link', 'imagely-free-spirit' ),
				    'section'     => 'header-colors',
				    'settings'    => 'imagely_menu_link',
				)
			)
		);

		/* Header: Menu link hover color */
		$wp_customize->add_setting(
			'imagely_menu_hover',
			array(
				'default'           => '#1CB495',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_menu_hover_control',
				array(
					'description' => __( 'Change the menu link hover color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Menu Link Hover', 'imagely-free-spirit' ),
				    'section'     => 'header-colors',
				    'settings'    => 'imagely_menu_hover',
				)
			)
		);

		/* Header: Submenu background color */
		$wp_customize->add_setting(
			'imagely_submenu_background',
			array(
				'default'           => '#83D3C9',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_submenu_background_control',
				array(
					'description' => __( 'Change the submenu background color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Submenu Background', 'imagely-free-spirit' ),
				    'section'     => 'header-colors',
				    'settings'    => 'imagely_submenu_background',
				)
			)
		);

		/* Header: Submenu link color */
		$wp_customize->add_setting(
			'imagely_submenu_link',
			array(
				'default'           => '#fff',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);
		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_submenu_link_control',
				array(
					'description' => __( 'Change the submenu link color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Submenu Link', 'imagely-free-spirit' ),
				    'section'     => 'header-colors',
				    'settings'    => 'imagely_submenu_link',
				)
			)
		);

		/* Header: Submenu link hover color */
		$wp_customize->add_setting(
			'imagely_submenu_hover',
			array(
				'default'           => '#eee',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_submenu_hover_control',
				array(
					'description' => __( 'Change the submenu link hover color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Submenu Link Hover', 'imagely-free-spirit' ),
				    'section'     => 'header-colors',
				    'settings'    => 'imagely_submenu_hover',
				)
			)
		);


	/* Add Mobile Header section to Custom Color Panel */
	$wp_customize->add_section( 'mobile-header-colors', array(
        'priority' => 10,
        'capability' => 'edit_theme_options',
        'theme_supports' => '',
        'title' => __( 'Mobile Header', 'imagely-free-spirit' ),
        'description' => 'Customize the header colors on smaller mobile devices like phones. To preview the appearance, grab the lower right corner of this browser window and reduce the window width until the responsive header appears.',
        'panel' => 'custom-colors',
    ) );

    	/* Mobile Header: Background color */
		$wp_customize->add_setting(
			'imagely_mobile_header_background',
			array(
				'default'           => '#FBFAFC',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_mobile_header_background_control',
				array(
					'description' => __( 'Change the header background color on mobile devices.', 'imagely-free-spirit' ),
				    'label'       => __( 'Mobile Header Background', 'imagely-free-spirit' ),
				    'section'     => 'mobile-header-colors',
				    'settings'    => 'imagely_mobile_header_background',
				)
			)
		);

		/* Mobile Header: Title color */
		$wp_customize->add_setting(
			'imagely_mobile_title_color',
			array(
				'default'           => '#888',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_mobile_title_color_control',
				array(
					'description' => __( 'Change the mobile title color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Mobile Title', 'imagely-free-spirit' ),
				    'section'     => 'mobile-header-colors',
				    'settings'    => 'imagely_mobile_title_color',
				)
			)
		);

		/* Mobile Header: Description color */
		$wp_customize->add_setting(
			'imagely_mobile_description_color',
			array(
				'default'           => '#AAA',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_description_color_control',
				array(
					'description' => __( 'Change the site description color on mobile devices (if visible).', 'imagely-free-spirit' ),
				    'label'       => __( 'Site Description', 'imagely-free-spirit' ),
				    'section'     => 'mobile-header-colors',
				    'settings'    => 'imagely_mobile_description_color',
				)
			)
		);
		
		/* Mobile Header: Menu link color */
		$wp_customize->add_setting(
			'imagely_mobile_menu_link',
			array(
				'default'           => '#888',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_mobile_menu_link_control',
				array(
					'description' => __( 'Change the menu/submenu link color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Menu/Submenu Link', 'imagely-free-spirit' ),
				    'section'     => 'mobile-header-colors',
				    'settings'    => 'imagely_mobile_menu_link',
				)
			)
		);

		/* Mobile Header: Menu hover color */
		$wp_customize->add_setting(
			'imagely_mobile_menu_hover',
			array(
				'default'           => '#1CB495',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_mobile_menu_hover_control',
				array(
					'description' => __( 'Change the menu/submenu hover color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Menu/Submenu Hover', 'imagely-free-spirit' ),
				    'section'     => 'mobile-header-colors',
				    'settings'    => 'imagely_mobile_menu_hover',
				)
			)
		);


	/* Add Content section to Custom Color Panel */
	$wp_customize->add_section( 'content-colors', array(
        'priority' => 10,
        'capability' => 'edit_theme_options',
        'theme_supports' => '',
        'title' => __( 'Content Area', 'imagely-free-spirit' ),
        'description' => '',
        'panel' => 'custom-colors',
    ) );

		/* Content: Link color */
		$wp_customize->add_setting(
			'imagely_link_color',
			array(
				'default'           => '#1CB495',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_link_color_control',
				array(
					'description' => __( 'Change the main link color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Link', 'imagely-free-spirit' ),
				    'section'     => 'content-colors',
				    'settings'    => 'imagely_link_color',
				)
			)
		);

		/* Content: Link hover color */
		$wp_customize->add_setting(
			'imagely_link_hover',
			array(
				'default'           => '#83D3C9',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_link_hover_control',
				array(
					'description' => __( 'Change the main link hover color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Link Hover', 'imagely-free-spirit' ),
				    'section'     => 'content-colors',
				    'settings'    => 'imagely_link_hover',
				)
			)
		);

		/* Other: Button color */
		$wp_customize->add_setting(
			'imagely_button_color',
			array(
				'default'           => '#1CB495',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_button_color_control',
				array(
					'description' => __( 'Change the button color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Button', 'imagely-free-spirit' ),
				    'section'     => 'content-colors',
				    'settings'    => 'imagely_button_color',
				)
			)
		);

    	/* Other: Button hover color */
		$wp_customize->add_setting(
			'imagely_button_hover',
			array(
				'default'           => '#36CEAF',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_button_hover_control',
				array(
					'description' => __( 'Change the button hover color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Button Hover', 'imagely-free-spirit' ),
				    'section'     => 'content-colors',
				    'settings'    => 'imagely_button_hover',
				)
			)
		);
	
	/* Add Footer section to Custom Color Panel */
	$wp_customize->add_section( 'footer-colors', array(
        'priority' => 10,
        'capability' => 'edit_theme_options',
        'theme_supports' => '',
        'title' => __( 'Footer', 'imagely-free-spirit' ),
        'description' => '',
        'panel' => 'custom-colors',
    ) );

		/* Footer: Display Imagely in Footer */
		/* @WP: Changed default from 1; changed description & label */
		$wp_customize->add_setting( 
			'powered_by_imagely', 
			array(
		    	'default' => 0,
		    	'sanitize_callback' => 'imagely_sanitize_checkbox',
		) );
		 
		$wp_customize->add_control( 
			'powered_by_imagely_control', 
			array(
				'description' => __( 'Not required, but appreciated.', 'imagely-free-spirit' ),
			    'label' => __( 'Display "Powered by Home Inspector Pro Websites" in footer?', 'imagely-free-spirit'),
			    'type' => 'checkbox',
			    'section' => 'footer-colors',
			    'settings' => 'powered_by_imagely',
		) );

		/* Footer: Background color */
		$wp_customize->add_setting(
			'imagely_footer_background',
			array(
				'default'           => '#FbFaFc',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_footer_background_control',
				array(
					'description' => __( 'Change the footer background color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Footer Background', 'imagely-free-spirit' ),
				    'section'     => 'footer-colors',
				    'settings'    => 'imagely_footer_background',
				)
			)
		);

		/* Footer: Widget title color */
		$wp_customize->add_setting(
			'imagely_footer_title',
			array(
				'default'           => '#2E3842',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_footer_title_control',
				array(
					'description' => __( 'Change the footer title color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Footer Title', 'imagely-free-spirit' ),
				    'section'     => 'footer-colors',
				    'settings'    => 'imagely_footer_title',
				)
			)
		);

		/* Footer: Text color */
		$wp_customize->add_setting(
			'imagely_footer_text',
			array(
				'default'           => '#555',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_footer_text_control',
				array(
					'description' => __( 'Change the footer text color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Footer Text', 'imagely-free-spirit' ),
				    'section'     => 'footer-colors',
				    'settings'    => 'imagely_footer_text',
				)
			)
		);

		/* Footer: Link color */
		$wp_customize->add_setting(
			'imagely_footer_link',
			array(
				'default'           => '#1CB495',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_footer_link_control',
				array(
					'description' => __( 'Change the footer link color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Footer Link', 'imagely-free-spirit' ),
				    'section'     => 'footer-colors',
				    'settings'    => 'imagely_footer_link',
				)
			)
		);

		/* Footer: Link hover color */
		$wp_customize->add_setting(
			'imagely_footer_hover',
			array(
				'default'           => '#1CB495',
				'sanitize_callback' => 'sanitize_hex_color',
			)
		);

		$wp_customize->add_control(
			new WP_Customize_Color_Control(
				$wp_customize,
				'imagely_footer_hover_control',
				array(
					'description' => __( 'Change the footer link hover color.', 'imagely-free-spirit' ),
				    'label'       => __( 'Footer Link Hover', 'imagely-free-spirit' ),
				    'section'     => 'footer-colors',
				    'settings'    => 'imagely_footer_hover',
				)
			)
		);	

}

/************************************************ 
 * 
 * WP CUSTOMIZER: Build and Output the CSS
 * 
 ************************************************/

add_action( 'wp_enqueue_scripts', 'imagely_css' );
function imagely_css() {

	$handle  = defined( 'CHILD_THEME_NAME' ) && CHILD_THEME_NAME ? sanitize_title_with_dashes( CHILD_THEME_NAME ) : 'child-theme';

	/* Retrieve the colors from customizer */
	$header_background = get_theme_mod( 'imagely_header_background', '#FbFaFc' );
	$title_color = get_theme_mod( 'imagely_title_color', '#888' );
	$description_color = get_theme_mod( 'imagely_description_color', '#888' );
	$menu_background = get_theme_mod( 'imagely_menu_background', '#FbFaFc' );
	$menu_link = get_theme_mod( 'imagely_menu_link', '#888' );
	$menu_hover = get_theme_mod( 'imagely_menu_hover', '#1CB495' );
	$submenu_background = get_theme_mod( 'imagely_submenu_background', '#83D3C9' );
	$submenu_link = get_theme_mod( 'imagely_submenu_link', '#fff' );
	$submenu_hover = get_theme_mod( 'imagely_submenu_hover', '#eee' );
	$mobile_header_background = get_theme_mod( 'imagely_mobile_header_background', '#FBFAFC' );
	$mobile_title_color = get_theme_mod( 'imagely_mobile_title_color', '#888' );
	$mobile_description_color = get_theme_mod( 'imagely_mobile_description_color', '#AAA' );
	$mobile_menu_link = get_theme_mod( 'imagely_mobile_menu_link', '#888' );
	$mobile_menu_hover = get_theme_mod( 'imagely_mobile_menu_hover', '#1CB495' );
	$link_color = get_theme_mod( 'imagely_link_color', '#1CB495' );
	$link_hover = get_theme_mod( 'imagely_link_hover', '#83D3C9' );
	$button_color = get_theme_mod( 'imagely_button_color', '#1CB495' );
	$button_hover = get_theme_mod( 'imagely_button_hover', '#36CEAF' );
	$footer_background = get_theme_mod( 'imagely_footer_background', '#FbFaFc' );
	$footer_title = get_theme_mod( 'imagely_footer_title', '#2E3842' );
	$footer_text = get_theme_mod( 'imagely_footer_text', '#555' );
	$footer_link = get_theme_mod( 'imagely_footer_link', '#1CB495' );
	$footer_hover = get_theme_mod( 'imagely_footer_hover', '#1CB495' );

	/* Build the CSS */
	$css = '';
	
	$css .= ( '#FbFaFc' !== $header_background ) ? sprintf( '
		.site-header {background-color:  %1$s; border-bottom: none; }' , $header_background ) : '';
	
	$css .= ( '#888' !== $title_color ) ? sprintf( '
		.site-title a, 
		.site-title a:hover {color:  %1$s;}' , $title_color ) : '';
	
	$css .= ( '#888' !== $description_color ) ? sprintf( '
		.site-description {color:  %1$s;}' , $description_color ) : '';
	
	$css .= ( '#FbFaFc' !== $menu_background ) ? sprintf( '
		.nav-primary {background-color:  %1$s;}' , $menu_background ) : '';
	
	$css .= ( '#888' !== $menu_link ) ? sprintf( '
		.nav-primary .genesis-nav-menu a {color:  %1$s;}' , $menu_link ) : '';
	
	$css .= ( '#1CB495' !== $menu_hover ) ? sprintf( '
		.nav-primary .genesis-nav-menu a:hover, 
		.nav-primary .genesis-nav-menu a:active {color:  %1$s;}' , $menu_hover ) : '';
	
	$css .= ( '#83D3C9' !== $submenu_background ) ? sprintf( '
		.genesis-nav-menu .sub-menu {background-color:  %1$s;}
		.genesis-nav-menu .sub-menu a {border-color: %1$s;}
		.genesis-nav-menu > li > ul::before {border-bottom-color: %1$s;}' , $submenu_background ) : '';

	$css .= ( '#fff' !== $submenu_link ) ? sprintf( '
		.nav-primary .genesis-nav-menu .sub-menu a,
		.site-header.transparent .nav-primary .genesis-nav-menu .sub-menu a {color:  %1$s;}' , $submenu_link ) : '';
	
	$css .= ( '#eee' !== $submenu_hover ) ? sprintf( '
		.nav-primary .genesis-nav-menu .sub-menu a:hover, 
		.nav-primary .genesis-nav-menu .sub-menu a:active,
		.site-header.transparent .nav-primary .genesis-nav-menu .sub-menu a:hover,
		.site-header.transparent .nav-primary .genesis-nav-menu .sub-menu a:active {color:  %1$s;}' , $submenu_hover ) : '';

	$css .= ( '#FBFAFC' !== $mobile_header_background ) ? sprintf( '
		@media only screen and (max-width: 1024px) {
			.site-header,
			.site-header.transparent,
			.nav-primary,
			.nav-primary .genesis-nav-menu .sub-menu,
			.nav-primary .responsive-menu, 
			.nav-primary .responsive-menu .sub-menu {background-color:  %1$s;}
		}' , $mobile_header_background ) : '';

	$css .= ( '#888' !== $mobile_title_color ) ? sprintf( '
		@media only screen and (max-width: 1024px) {
			.site-title a, 
			.site-title a:hover,
			.site-header.transparent .site-title a,
			.site-header.transparent .site-title a:hover {color:  %1$s;}
		}' , $mobile_title_color ) : '';
	
	$css .= ( '#AAA' !== $mobile_description_color ) ? sprintf( '
		@media only screen and (max-width: 1024px) {
			.site-description, 
			.site-header.transparent .site-description {color:  %1$s;}
		}' , $mobile_description_color ) : '';

	$css .= ( '#888' !== $mobile_menu_link ) ? sprintf( '
		.responsive-menu-icon::before {color:  %1$s;}
		@media only screen and (max-width: 1024px) {
			.nav-primary .genesis-nav-menu a,
			.nav-primary .genesis-nav-menu .sub-menu a,
			.nav-primary .responsive-men a,
			.nav-primary .current-menu-item > a
			.site-header.transparent .nav-primary .genesis-nav-menu a,
			.site-header.transparent .nav-primary .genesis-nav-menu .sub-menu a,
			.site-header.transparent .responsive-menu a,
			.site-header.transparent .responsive-menu .sub-menu a
			.responsive-menu-icon::before,
			.nav-primary .responsive-menu .menu-item-has-children::before {color:  %1$s;}
		}' , $mobile_menu_link ) : '';
	
	$css .= ( '#1CB495' !== $mobile_menu_hover ) ? sprintf( '
		@media only screen and (max-width: 1024px) {
			.nav-primary .genesis-nav-menu a:hover, 
			.nav-primary .genesis-nav-menu a:active,
			.nav-primary .genesis-nav-menu .sub-menu a:hover, 
			.nav-primary .genesis-nav-menu .sub-menu a:active,
			.nav-primary .responsive-menu a:hover,
			.nav-primary .responsive-menu .current-menu-item > a,
			.nav-primary .responsive-menu .sub-menu a:hover,
			.nav-primary .responsive-menu .sub-menu a:focus,
			.site-header.transparent .genesis-nav-menu a:hover, 
			.site-header.transparent .genesis-nav-menu a:active, 
			.site-header.transparent .genesis-nav-menu .sub-menu a:hover, 
			.site-header.transparent .genesis-nav-menu .sub-menu a:active {color:  %1$s;}

			.site-header, 
			.site-header.transparent {border-bottom-color: %1$s; }
		}' , $mobile_menu_hover ) : '';

	$css .= ( '#1CB495' !== $link_color ) ? sprintf( '
		a,
		.author-box a,
		.archive-description a {color:  %1$s;}' , $link_color ) : '';
	
	$css .= ( '#83D3C9' !== $link_hover ) ? sprintf( '
		a:hover, 
		a:focus,
		.author-box a:hover,
		.author-box a:focus,
		.archive-description a:hover,
		.archive-description a:focus {color:  %1$s;}' , $link_hover ) : '';
	
	$css .= ( '#FbFaFc' !== $footer_background ) ? sprintf( '
		.footer-widgets, 
		.site-footer {background-color:  %1$s;}' , $footer_background ) : '';
	
	$css .= ( '#2E3842' !== $footer_title ) ? sprintf( '
		.footer-widgets .widget-title {color:  %1$s;}' , $footer_title ) : '';
	
	$css .= ( '#555' !== $footer_text ) ? sprintf( '
		.footer-widgets, 
		.site-footer {color:  %1$s;}' , $footer_text ) : '';
	
	$css .= ( '#1CB495' !== $footer_link) ? sprintf( '
		.footer-widgets a, 
		.footer-widgets .genesis-nav-menu a,
		.footer-widgets .entry-title a,
		.site-footer a,
		.site-footer .genesis-nav-menu a {color:  %1$s;}' , $footer_link ) : '';
		
	$css .= ( '#1CB495' !== $footer_hover ) ? sprintf( '
		.footer-widgets a:hover, 
		.footer-widgets a:active, 
		.footer-widgets .genesis-nav-menu a:active,
		.footer-widgets .genesis-nav-menu a:hover,
		.footer-widgets .entry-title a:hover,
  		.footer-widgets .entry-title a:focus,
		.site-footer a:hover, 
		.site-footer a:hover,
		.site-footer .genesis-nav-menu a:active,
		.site-footer .genesis-nav-menu a:hover {color:  %1$s;}' , $footer_hover ) : '';
	
	$css .= ( '#1CB495' !== $button_color ) ? sprintf( '
		button,
		input[type="button"],
		input[type="reset"],
		input[type="submit"],
		.button,
		.content .widget .textwidget a.button,
		.entry-content a.button,
		.entry-content a.more-link,
		.footer-widgets button,
		.footer-widgets input[type="button"],
		.footer-widgets input[type="reset"],
		.footer-widgets input[type="submit"],
		.footer-widgets .button,
		.footer-widgets .entry-content a.more-link,
		.content .front-page-1 .widget a.button,
		.content .front-page-1 .widget .textwidget a.button,
		.front-page-1 button,
		.front-page-1 input[type="button"],
		.front-page-1 input[type="reset"],
		.front-page-1 input[type="submit"],
		.front-page-1 .entry-content a.button,
		.front-page-1 .entry-content a.more-link,
		.nav-primary li.highlight > a,
		.archive-pagination li a:hover,
		.archive-pagination li a:focus,
		.front-page .content .fa {background-color:  %1$s;}' , $button_color ) : '';
	
	$css .= ( '#36CEAF' !== $button_hover ) ? sprintf( '
		button:hover,
		button:focus,
		input:hover[type="button"],
		input:focus[type="button"],
		input:hover[type="reset"],
		input:focus[type="reset"],
		input:hover[type="submit"],
		input:focus[type="submit"],
		.button:hover,
		.button:focus,
		.content .widget .textwidget a.button:hover,
		.content .widget .textwidget a.button:focus,
		.entry-content a.button:hover,
		.entry-content a.button:focus,
		.entry-content a.more-link:hover,
		.entry-content a.more-link:focus,
		.footer-widgets button:hover,
		.footer-widgets button:focus,
		.footer-widgets input:hover[type="button"],
		.footer-widgets input:focus[type="button"],
		.footer-widgets input:hover[type="reset"],
		.footer-widgets input:focus[type="reset"],
		.footer-widgets input:hover[type="submit"],
		.footer-widgets input:focus[type="submit"],
		.footer-widgets .button:hover,
		.footer-widgets .button:focus,
		.footer-widgets .entry-content a.more-link:hover,
		.footer-widgets .entry-content a.more-link:focus,
		.content .front-page-1 .widget a.button:hover,
		.content .front-page-1 .widget a.button:focus,
		.content .front-page-1 .widget .textwidget a.button:hover,
		.content .front-page-1 .widget .textwidget a.button:focus,
		.front-page-1 button:hover,
		.front-page-1 button:focus,
		.front-page-1 input:hover[type="button"],
		.front-page-1 input:focus[type="button"],
		.front-page-1 input:hover[type="reset"],
		.front-page-1 input:focus[type="reset"],
		.front-page-1 input:hover[type="submit"],
		.front-page-1 input:focus[type="submit"],
		.front-page-1 .entry-content a.button:hover,
		.front-page-1 .entry-content a.button:focus,
		.front-page-1 .entry-content a.more-link:hover,
		.front-page-1 .entry-content a.more-link:focus,
		.nav-primary li.highlight > a:hover,
		.nav-primary li.highlight > a:focus,
		.archive-pagination li a,
		.archive-pagination .active a {background-color:  %1$s;}' , $button_hover ) : '';


	/* Output the CSS */
	if( $css ){
		wp_add_inline_style( $handle, $css );
	}

}

/* Callback to sanitize checkbox setting */
function imagely_sanitize_checkbox( $checked ) {
	return ( ( isset( $checked ) && true == $checked ) ? true : false );
}